/*
xAffect - Middleware for Physiological Sensors and Affective Signal Processing
Copyright (c) 2011-2012 FZI Research Center for Information Technology, Germany

This file is part of the xAffect middleware. For more information, see http://www.xaffect.org.

This software is licensed under the Original BSD License. You should have received a copy of the Original BSD License along with xAffect. If not, see http://www.xaffect.org/license/.
 */
package org.xaffect;

import java.io.File;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.net.URL;
import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Date;
import java.util.Vector;
import java.util.logging.FileHandler;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.logging.SimpleFormatter;

import org.xaffect.config.Constants;
import org.xaffect.config.ConstantsDesktop;
import org.xaffect.control.Control;
import org.xaffect.control.gui.XAffectWindowControl;
import org.xaffect.control.net.XAffectNetworkControl;
import org.xaffect.core.LifeCycleManager;
import org.xaffect.desktop.classloader.SetupFinder;
import org.xaffect.desktop.config.ConfigFile;
import org.xaffect.desktop.config.DesktopConstants;

/**
 * Represents the desktop-version of {@link XAffect}, using the
 * configuration in {@link ConstantsDesktop#CONFIG_FILE}.
 */
public class XAffectDesktop
{
	/**
	 * Creates a new {@link XAffectDesktop}.
	 */
	public XAffectDesktop()
	{
		Logger logger = java.util.logging.Logger.getLogger("org.xaffect");
		try
		{
			SimpleFormatter formatter = new SimpleFormatter();

			// set logging-file
			SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd_HH-mm-ss");
			String loggingPath = "log" + File.separator + Constants.LOGGER_FILE_PREFIX + dateFormat.format(new Date())
					+ Constants.LOGGER_FILE_EXTENSION;
			FileHandler fh = new FileHandler(loggingPath);
			fh.setFormatter(formatter);
			logger.addHandler(fh);

			// set logging-level info
			Level logLevel = Level.INFO;
			logger.setLevel(logLevel);
		}
		catch (Exception e)
		{
			e.printStackTrace();
			logger.log(Level.SEVERE, e.toString());
		}
		
		LifeCycleManager manager = new LifeCycleManager(logger);
		ConfigFile config = new ConfigFile(manager.getLogger());

		// get class-name from .properties
		String controllerClass = config.getProperty(DesktopConstants.PROPERTY_CONTROL_INTERFACE);
		if (controllerClass != null && !controllerClass.isEmpty())
		{
			try
			{
				// load class via classloader
				@SuppressWarnings("unchecked")
				Class<Control> controlClass = (Class<Control>) XAffect.class.getClassLoader()
						.loadClass(controllerClass);
				// instantiate control
				Control control = this.getControl(manager, controlClass);
				// start xAffect
				if (control != null)
				{
					new XAffect(control);
				}
				else
				{
					manager.getLogger().log(
							Level.SEVERE,
							"Failed to load class \"" + controllerClass + "\". Please check the property "
									+ DesktopConstants.PROPERTY_CONTROL_INTERFACE + " in "
									+ DesktopConstants.CONFIG_FILE + ".");
				}
			}
			catch (ClassNotFoundException e)
			{
				manager.getLogger().log(
						Level.SEVERE,
						"The class " + controllerClass + " cannot be found in xAffect. Please check the property "
								+ DesktopConstants.PROPERTY_CONTROL_INTERFACE + " in " + DesktopConstants.CONFIG_FILE
								+ ".");
			}
		}
		else
		{
			// controllerClass is null or empty
			manager.getLogger().log(
					Level.WARNING,
					"The property " + DesktopConstants.PROPERTY_CONTROL_INTERFACE + " is not set in "
							+ DesktopConstants.CONFIG_FILE + ".");
		}
	}

	/**
	 * Instantiates the {@link Control} given as a {@link Class} in
	 * controlClass.
	 * 
	 * @param manager
	 *            {@link LifeCycleManager} to instantiate the controlClass for.
	 * @param controlClass
	 *            {@link Class} representing the {@link Control} to be
	 *            instantiated.
	 * @return An instance of the {@link Control} or null if the control could
	 *         not be instanciated.
	 */
	private Control getControl(LifeCycleManager manager, Class<Control> controlClass)
	{
		Control control = null;

		try
		{
			// check whether controlClass represents XAffectNetworkControl
			if (controlClass.getName().equals(XAffectNetworkControl.class.getName()))
			{
				control = this.getXAffectNetworkControl(manager);
			}
			// check whether controlClass represents XAffectWindowControl
			else if (controlClass.getName().equals(XAffectWindowControl.class.getName()))
			{
				control = this.getXAffectWindowControl(manager);
			}
			// try to create the Control by using the standard-constructor
			else
			{
				try
				{
					// try to find standard-constructor
					// Control(LifeCycleManager)
					Constructor<Control> controlConstructor = controlClass.getConstructor(LifeCycleManager.class);
					control = controlConstructor.newInstance(manager);
				}
				catch (NoSuchMethodException e)
				{
					manager.getLogger().log(
							Level.SEVERE,
							"The instantiation of " + controlClass.getSimpleName()
									+ " was not successful.\r Please modify " + this.getClass().getSimpleName()
									+ ".getControl() if you are trying to use a custom Control implementing"
									+ " a constructor other than Control(LifeCycleManager).");
				}
			}
		}
		catch (Exception e)
		{
			manager.getLogger().log(
					Level.SEVERE,
					"The instantiation of " + controlClass.getSimpleName() + " was not successful.\r"
							+ e.getLocalizedMessage());
		}

		return control;
	}

	/**
	 * Instantiates a {@link XAffectNetworkControl} for the current context of
	 * {@link XAffectDesktop}.
	 * 
	 * @param manager
	 *            {@link LifeCycleManager} to instantiate the control for.
	 * @return a new Instance of {@link XAffectNetworkControl}.
	 */
	private XAffectNetworkControl getXAffectNetworkControl(LifeCycleManager manager)
	{
		// get configuration on port
		ConfigFile config = new ConfigFile(manager.getLogger());
		int port = ConstantsDesktop.PROPERTY_CONTROL_NETWORK_PORT_DEFAULT;
		// check for port-declaration in config
		if (config.containsKey(ConstantsDesktop.PROPERTY_CONTROL_NETWORK_PORT))
		{
			try
			{
				port = Integer.parseInt((String) config.get(ConstantsDesktop.PROPERTY_CONTROL_NETWORK_PORT));
			}
			catch (NumberFormatException e)
			{
				// log warning
				manager.getLogger().log(
						Level.WARNING,
						"Found configuration for " + ConstantsDesktop.PROPERTY_CONTROL_NETWORK_PORT
								+ " but cannot parse value to integer.\rPlease check the property "
								+ DesktopConstants.PROPERTY_CONTROL_NETWORK_PORT + " in "
								+ DesktopConstants.CONFIG_FILE + ".");
			}
		}

		return new XAffectNetworkControl(manager, port);
	}

	/**
	 * Instantiates a {@link XAffectWindowControl} for the current context of
	 * {@link XAffectDesktop}.
	 * 
	 * @param manager
	 *            {@link LifeCycleManager} to instantiate the control for.
	 * @return a new Instance of {@link XAffectWindowControl}.
	 */
	private XAffectWindowControl getXAffectWindowControl(LifeCycleManager manager)
	{
		// get configuration
		ConfigFile config = new ConfigFile(manager.getLogger());

		// get available studies
		String[] availableStudies = { "NONE" };
		if (config.containsKey(ConstantsDesktop.PROPERTY_CONTROL_WINDOW_AVAILABLE_STUDY_IDS))
		{
			availableStudies = config.getPropertyArray(ConstantsDesktop.PROPERTY_CONTROL_WINDOW_AVAILABLE_STUDY_IDS);
		}

		// get available institutions
		String[] availableInstitutions = { "NONE" };
		if (config.containsKey(ConstantsDesktop.PROPERTY_CONTROL_WINDOW_AVAILABLE_INSTITUTION_IDS))
		{
			availableInstitutions = config
					.getPropertyArray(ConstantsDesktop.PROPERTY_CONTROL_WINDOW_AVAILABLE_INSTITUTION_IDS);
		}

		// get setups
		String[] setups = {};

		// get available setups
		URL xAffectPath = XAffectDesktop.class.getProtectionDomain().getCodeSource().getLocation();
		String rootDir = xAffectPath.getPath().substring(0, xAffectPath.getPath().lastIndexOf("/"));
		rootDir = rootDir.replaceAll("%20", " ");
		Vector<String> availableSetups = SetupFinder.getAllSetupsIn(new File(rootDir));

		// get excluded setups
		if (config.containsKey(ConstantsDesktop.PROPERTY_CONTROL_WINDOW_EXCLUDE_SETUPS))
		{
			Vector<String> excludedSetups = new Vector<String>(Arrays.asList(config.getProperty(
					ConstantsDesktop.PROPERTY_CONTROL_WINDOW_EXCLUDE_SETUPS).split(";")));

			// filter setups
			Vector<String> filteredSetups = new Vector<String>();
			for (String foundClass : availableSetups)
			{
				// filter Setup-interface && excluded setups out of list
				if (!foundClass.equalsIgnoreCase("org.xaffect.setup.Setup") && !excludedSetups.contains(foundClass))
				{
					filteredSetups.add(foundClass);
				}
			}
			setups = new String[filteredSetups.size()];
			filteredSetups.toArray(setups);
		}
		else
		{
			// no setups excluded
			setups = new String[availableSetups.size()];
			availableSetups.toArray(setups);
		}

		// instantiate control
		return new XAffectWindowControl(manager, setups, availableStudies, availableInstitutions);
	}
}
