/*
xAffect - Middleware for Physiological Sensors and Affective Signal Processing
Copyright (c) 2011-2012 FZI Research Center for Information Technology, Germany

This file is part of the xAffect middleware. For more information, see http://www.xaffect.org.

This software is licensed under the Original BSD License. You should have received a copy of the Original BSD License along with xAffect. If not, see http://www.xaffect.org/license/.
 */

package org.xaffect.components.sink.dataviewer;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.widgets.TabItem;
import org.xaffect.components.sink.dataviewer.resources.chartviewer.ChartEventViewer;
import org.xaffect.components.sink.dataviewer.resources.chartviewer.ChartSignalViewer;
import org.xaffect.components.sink.dataviewer.resources.chartviewer.ChartValueViewer;
import org.xaffect.config.ConstantsDesktop;
import org.xaffect.core.data.DataEntry;
import org.xaffect.core.data.DataEvent;
import org.xaffect.core.data.DataSignal;
import org.xaffect.core.data.DataValue;

/**
 * GUI displaying {@link DataEntry}s in a chart-diagram.
 */
public class DataViewer
{
	/**
	 * Defining the {@link DataViewer} to display {@link DataSignal}s.
	 */
	public static final int SIGNAL = 1;
	/**
	 * Defining the {@link DataViewer} to display {@link DataValue}s.
	 */
	public static final int VALUE = 2;
	/**
	 * Defining the {@link DataViewer} to display {@link DataEvent}s.
	 */
	public static final int EVENT = 4;

	/**
	 * Describing the appearance of the {@link DataViewer}. Can be one of the
	 * Values {@link #SIGNAL}, {@link #VALUE} or {@link #EVENT}, or the product
	 * of their bitwise-or.
	 */
	private int displayStyle;

	private boolean autoAdjust = true;
	private double minValue = -100.0;
	private double maxValue = 100.0;
	private long timeToDisplay = 7000;

	Display display;

	// widgets
	private Shell shell;
	private TabFolder tabFolder;

	// viewers
	private ChartSignalViewer chartSignalViewer;
	private ChartEventViewer chartEventViewer;
	private ChartValueViewer chartValueViewer;

	/**
	 * Creates a new {@link DataViewer}
	 * 
	 * @param display
	 *            The UI-{@link Display}.
	 * @param displayStyle
	 *            Style describing the appearance of the {@link DataViewer}.
	 */
	public DataViewer(int displayStyle)
	{
		this.display = Display.getDefault();
		this.displayStyle = displayStyle;
		this.generateWidgets();
	}

	/**
	 * Creates a new {@link DataViewer}
	 * 
	 * @param display
	 *            The UI-{@link Display}.
	 * @param displayStyle
	 *            Style describing the appearance of the {@link DataViewer}.
	 * @param autoAdjust
	 *            Determines whether the auto-adjust-feature is to be set or not
	 *            by default.
	 * @param minValue
	 *            Sets the standard-value of minValue.
	 * @param maxValue
	 *            Sets the standard value of maxValue.
	 * @param timeToDispaly
	 *            Sets the standard time to display on canvas (in ms).
	 */
	public DataViewer(int displayStyle, boolean autoAdjust, double minValue, double maxValue, long timeToDispaly)
	{
		this.display = Display.getDefault();
		this.displayStyle = displayStyle;
		this.autoAdjust = autoAdjust;
		this.minValue = minValue;
		this.maxValue = maxValue;
		this.timeToDisplay = timeToDispaly;
		this.generateWidgets();
	}

	/**
	 * Generates widgets depending on the styles set in {@link #displayStyle}.
	 */
	private void generateWidgets()
	{
		this.shell = new Shell(this.display, SWT.SHELL_TRIM);
		this.shell.setText("xAffect AdvancedViewer");
		this.shell.setSize(800, 600);
		this.shell.setImage(new Image(display, ConstantsDesktop.IMAGE_PATH + "xAffect.ico"));
		this.shell.setLayout(new FillLayout());

		this.tabFolder = new TabFolder(this.shell, SWT.NONE);
		this.tabFolder.setLocation(this.shell.getClientArea().x, this.shell.getClientArea().y);

		// set up value-view
		if (this.displayStyle != 0)
		{
			switch (this.displayStyle)
			{
				case SIGNAL:
					this.generateSignalTab();
					break;
				case EVENT:
					this.generateEventTab();
					break;
				case VALUE:
					this.generateValueTab();
					break;
				case SIGNAL | EVENT:
					this.generateSignalTab();
					this.generateEventTab();
					break;
				case SIGNAL | VALUE:
					this.generateSignalTab();
					this.generateValueTab();
					break;
				case SIGNAL | EVENT | VALUE:
					this.generateSignalTab();
					this.generateEventTab();
					this.generateValueTab();
					break;
				case EVENT | VALUE:
					this.generateEventTab();
					this.generateValueTab();
					break;
				default:
					break;
			}
		}
	}

	/**
	 * Generates a TabItem containing a {@link ChartSignalViewer}.
	 */
	private void generateSignalTab()
	{
		TabItem tabValue = new TabItem(this.tabFolder, SWT.NONE);
		tabValue.setText("Signal View");
		this.chartSignalViewer = new ChartSignalViewer(this.tabFolder, this.autoAdjust, this.minValue, this.maxValue,
				this.timeToDisplay);
		tabValue.setControl(this.chartSignalViewer);
	}

	/**
	 * Generates a TabItem containing a {@link ChartValueViewer}.
	 */
	private void generateValueTab()
	{
		TabItem tabValue = new TabItem(this.tabFolder, SWT.NONE);
		tabValue.setText("Value View");
		this.chartValueViewer = new ChartValueViewer(this.tabFolder, this.autoAdjust, this.minValue, this.maxValue,
				this.timeToDisplay);
		tabValue.setControl(this.chartValueViewer);
	}

	/**
	 * Generates a TabItem containing a {@link ChartEventViewer}.
	 */
	private void generateEventTab()
	{
		TabItem tabValue = new TabItem(this.tabFolder, SWT.NONE);
		tabValue.setText("Event View");
		this.chartEventViewer = new ChartEventViewer(this.tabFolder);
		tabValue.setControl(this.chartEventViewer);
	}

	/**
	 * Draws given {@link DataEntry} to every component that can display it.
	 * 
	 * @param data
	 *            {@link DataEntry} to be drawn
	 */
	public void draw(final DataEntry data)
	{
		this.display.asyncExec(new Runnable()
		{
			
			@Override
			public void run()
			{
				if (data instanceof DataSignal)
				{
					if (DataViewer.this.chartSignalViewer != null && !DataViewer.this.chartSignalViewer.isDisposed())
					{
						DataViewer.this.chartSignalViewer.draw((DataSignal) data);
					}
				}
				else if (data instanceof DataEvent)
				{
					if (DataViewer.this.chartEventViewer != null && !DataViewer.this.chartEventViewer.isDisposed())
					{
						DataViewer.this.chartEventViewer.draw((DataEvent) data);
					}
				}
				else if (data instanceof DataValue)
				{
					if (DataViewer.this.chartValueViewer != null && !DataViewer.this.chartValueViewer.isDisposed())
					{
						DataViewer.this.chartValueViewer.draw((DataValue) data);
					}
				}
			}
		});
	}

	/**
	 * Opens the {@link DataViewer} in a new {@link Shell}.
	 */
	public void open()
	{
		Display.getDefault().asyncExec(new Runnable()
		{
			@Override
			public void run()
			{
				if (DataViewer.this.shell != null)
				{
					DataViewer.this.shell.open();
				}
			}
		});
	}

	/**
	 * Closes the {@link DataViewer}.
	 */
	public void close()
	{
		Display.getDefault().asyncExec(new Runnable()
		{
			@Override
			public void run()
			{
				if (DataViewer.this.shell != null && !DataViewer.this.shell.isDisposed())
				{
					DataViewer.this.shell.close();
					DataViewer.this.shell.dispose();
				}
			}
		});
	}
}