/*
xAffect - Middleware for Physiological Sensors and Affective Signal Processing
Copyright (c) 2011-2012 FZI Research Center for Information Technology, Germany

This file is part of the xAffect middleware. For more information, see http://www.xaffect.org.

This software is licensed under the Original BSD License. You should have received a copy of the Original BSD License along with xAffect. If not, see http://www.xaffect.org/license/.
 */

package org.xaffect.components.sink.dataviewer;

import java.util.HashMap;
import java.util.Vector;
import java.util.logging.Level;

import org.eclipse.swt.widgets.Display;
import org.xaffect.core.LifeCycleManager;
import org.xaffect.core.components.AbstractSink;
import org.xaffect.core.components.InstantiationException;
import org.xaffect.core.data.DataDescription;
import org.xaffect.core.data.DataEntry;
import org.xaffect.core.data.DataEvent;
import org.xaffect.core.data.DataSignal;
import org.xaffect.core.data.DataType;
import org.xaffect.core.data.DataValue;

/**
 * Component that is able to display the values of the three {@link DataEntry}
 * -types
 * <ul>
 * <li>{@link DataEvent}</li>
 * <li>{@link DataSignal}</li>
 * <li>{@link DataValue}</li>
 * </ul>
 * in a chart-diagram.
 */
public class SinkDataViewer extends AbstractSink
{
	/**
	 * Specifies the content class. Used for dispatching-uses only, but
	 * <b>[mandatory]</b> anyway.
	 */
	public static final String CONFIG_CONTENT_CLASS = "CONTENTCLASS";
	/**
	 * Specifies the number of channels, incoming data will contain.
	 * <b>[mandatory]</b>
	 */
	public static final String CONFIG_NUMBER_OF_CHANNELS = "NUMBEROFCHANNELS";
	/**
	 * Specifies the frequency with which incoming data will be sent.
	 * <b>[mandatory]</b>
	 */
	public static final String CONFIG_SAMPLE_RATE = "SAMPLERATE";
	/**
	 * Specifies the "design" of the {@link DataViewer}. The property can
	 * contain the following values:
	 * <ul>
	 * <li>{@link SinkDataViewer}.{@link #SIGNAL}</li>
	 * <li>{@link SinkDataViewer}.{@link #EVENT}</li>
	 * <li>{@link SinkDataViewer}.{@link #VALUE}</li>
	 * </ul>
	 * The values can be combined with a logical or in the following manner: <br />
	 * <code>
	 * cfgDataViewer.put(SinkDataViewer.CONFIG_STYLE,
				Integer.toString(SinkDataViewer.SIGNAL | SinkDataViewer.EVENT | SinkDataViewer.VALUE));
	 * </code>
	 * 
	 * <b>[mandatory]</b>
	 */
	public static final String CONFIG_STYLE = "STYLE";
	/**
	 * Determines whether the auto-adjust-feature is enabled by default ("true"
	 * or "false"). <b>[optional, defaults to "true"]</b>
	 */
	public static final String CONFIG_AUTO_ADJUST = "AUTOADJUST";
	/**
	 * Sets the lower border-value for signal- and value-display. <b>[optional,
	 * defaults to -100.0]</b>
	 */
	public static final String CONFIG_MIN_VALUE = "MINVALUE";
	/**
	 * Sets the upper border-value for signal- and value-display. <b>[optional,
	 * defaults to 100.0]</b>
	 */
	public static final String CONFIG_MAX_VALUE = "MAXVALUE";
	/**
	 * Sets the default time to display on canvas (in ms). <b>[optional,
	 * defaults to 7000ms]</b>
	 */
	public static final String CONFIG_TIME_TO_DISPLAY = "TIMETODISPLAY";
	/**
	 * @see {@link DataViewer#SIGNAL}
	 */
	public static final int SIGNAL = DataViewer.SIGNAL;
	/**
	 * @see {@link DataViewer#EVENT}
	 */
	public static final int EVENT = DataViewer.EVENT;
	/**
	 * @see {@link DataViewer#VALUE}
	 */
	public static final int VALUE = DataViewer.VALUE;

	/**
	 * {@link Vector} of {@link DataDescription}s describing the data the
	 * component is able to process.
	 */
	private DataDescription inputDescription;
	/**
	 * The {@link SinkDataViewer}'s {@link DataViewer}.
	 */
	private DataViewer viewer;

	// config
	private int displayStyle;
	private boolean autoAdjust = true;
	private double minValue = -100.0;
	private double maxValue = 100.0;
	private long timeToDisplay = 7000;

	public SinkDataViewer(LifeCycleManager manager, HashMap<String, String> configuration) throws InstantiationException
	{
		super(manager, configuration);
		
		if (configuration != null)
		{
			// check for contentclass
			if (configuration.containsKey(CONFIG_CONTENT_CLASS))
			{
				this.inputDescription = new DataDescription(configuration.get(CONFIG_CONTENT_CLASS), DataType.ANY);

				try
				{
					// check for sample-rate
					if (configuration.containsKey(CONFIG_SAMPLE_RATE))
					{
						this.inputDescription.setSampleRate(Double.parseDouble(configuration.get(CONFIG_SAMPLE_RATE)));
					}
					else
					{
						super.getLifeCycleManager()
								.getLogger()
								.log(Level.WARNING,
										this.getClass().getSimpleName() + ": No samplerate set. Default is 1.0.");

						this.inputDescription.setSampleRate(1.0);
					}
					// check for number-of-channels
					if (configuration.containsKey(CONFIG_NUMBER_OF_CHANNELS))
					{
						this.inputDescription.setNumberOfChannels(Integer.parseInt(configuration
								.get(CONFIG_NUMBER_OF_CHANNELS)));
					}
					else
					{
						super.getLifeCycleManager()
								.getLogger()
								.log(Level.WARNING,
										this.getClass().getSimpleName()
												+ ": No info about number of channels set. Default is 1.");

						this.inputDescription.setNumberOfChannels(1);
					}
				}
				catch (NumberFormatException ex)
				{
					super.getLifeCycleManager()
							.getLogger()
							.log(Level.WARNING,
									this.getClass().getSimpleName()
											+ ": Could not parse parameters to int, respectively double.");
				}

				// check for event-style
				if (configuration.containsKey(CONFIG_STYLE))
				{
					try
					{
						this.displayStyle = Integer.parseInt(configuration.get(CONFIG_STYLE));
					}
					catch (NumberFormatException ex)
					{
						super.getLifeCycleManager()
								.getLogger()
								.log(Level.WARNING,
										this.getClass().getSimpleName()
												+ ": Could not parse \"CONFIG_STYLE_EVENT\" to int.");
					}
				}
			}
			else
			{
				throw new InstantiationException("No contentclass set.");
			}
		}
		else
		{
			throw new InstantiationException("No configuration set.");
		}


		if (configuration.containsKey(CONFIG_AUTO_ADJUST))
		{
			String enabled = configuration.get(CONFIG_AUTO_ADJUST);
			if (enabled.equalsIgnoreCase("true"))
			{
				this.autoAdjust = true;
			}
			else if (enabled.equalsIgnoreCase("false"))
			{
				this.autoAdjust = false;
			}
			else
			{
				throw new InstantiationException("Cannot parse \"CONFIG_AUTO_ADJUST\" to boolean.");
			}
		}

		if (configuration.containsKey(CONFIG_MIN_VALUE))
		{
			try
			{
				this.minValue = Double.parseDouble(configuration.get(CONFIG_MIN_VALUE));
			}
			catch (NumberFormatException ex)
			{
				throw new InstantiationException("Cannot parse \"CONFIG_MIN_VALUE\" to double.");
			}
		}

		if (configuration.containsKey(CONFIG_MAX_VALUE))
		{
			try
			{
				this.maxValue = Double.parseDouble(configuration.get(CONFIG_MAX_VALUE));
			}
			catch (NumberFormatException ex)
			{
				throw new InstantiationException("Cannot parse \"CONFIG_MAX_VALUE\" to double.");
			}
		}

		if (configuration.containsKey(CONFIG_TIME_TO_DISPLAY))
		{
			try
			{
				this.timeToDisplay = Long.parseLong(configuration.get(CONFIG_TIME_TO_DISPLAY));
			}
			catch (NumberFormatException ex)
			{
				throw new InstantiationException("Cannot parse \"CONFIG_TIME_TO_DISPLAY\" to long.");
			}
		}
	}

	@Override
	public DataDescription[] getInputDescription()
	{
		return new DataDescription[] { this.inputDescription };
	}

	@Override
	protected void configure()
	{
		if (this.viewer == null)
		{
			Display.getDefault().asyncExec(new Runnable()
			{
				@Override
				public void run()
				{
					SinkDataViewer.this.viewer = new DataViewer(SinkDataViewer.this.displayStyle,
							SinkDataViewer.this.autoAdjust, SinkDataViewer.this.minValue, SinkDataViewer.this.maxValue,
							SinkDataViewer.this.timeToDisplay);
					SinkDataViewer.this.viewer.open();
				}
			});
		}
	}

	@Override
	protected void train()
	{
		// do nothing
	}

	@Override
	protected void processData(DataEntry entry)
	{
		if (this.viewer != null)
		{
			this.viewer.draw(entry);
		}
	}

	@Override
	protected void measure()
	{
		// nothing to do here
	}

	@Override
	protected void stop()
	{
		this.viewer.close();
	}
}
