/*
xAffect - Middleware for Physiological Sensors and Affective Signal Processing
Copyright (c) 2011-2012 FZI Research Center for Information Technology, Germany

This file is part of the xAffect middleware. For more information, see http://www.xaffect.org.

This software is licensed under the Original BSD License. You should have received a copy of the Original BSD License along with xAffect. If not, see http://www.xaffect.org/license/.
 */

package org.xaffect.components.sink.dataviewer.resources.chartviewer;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseMoveListener;
import org.eclipse.swt.events.MouseWheelListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Cursor;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.layout.FormLayout;
import org.eclipse.swt.widgets.Canvas;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.ToolBar;
import org.eclipse.wb.swt.SWTResourceManager;
import org.xaffect.components.sink.dataviewer.resources.DataViewer;
import org.xaffect.components.sink.dataviewer.resources.common.DropDownMenu;
import org.xaffect.components.sink.dataviewer.resources.common.Option;
import org.xaffect.components.sink.dataviewer.resources.common.OptionSelectedListener;
import org.xaffect.components.sink.dataviewer.resources.common.ValueScale;
import org.xaffect.components.sink.dataviewer.resources.common.ValueScaleListener;
import org.xaffect.core.data.DataEntry;

/**
 * Generic SWT-Composite able to display data as a chart-diagram.<br />
 * The type of data to display is specified by its generic parameter.
 * 
 * @param <DataType>
 *            specifying the type of data the {@link ChartDataViewer} is able to
 *            display. The DataType has to be derivable from {@link DataEntry}.
 */
public abstract class ChartDataViewer<DataType extends DataEntry> extends DataViewer<DataType>
{
	/**
	 * The canvas' background-color.
	 */
	private Color COLOR_BACKGROUND = SWTResourceManager.getColor(75, 75, 75);
	/**
	 * Specifies whether the viewer is supposed to show a scale for the values.
	 */
	private boolean displayScale;
	/**
	 * Specifies whether the viewer is supposed to show a caption describing the
	 * values.
	 */
	private boolean displayCaption;
	/**
	 * Specifies whether the viewer is supposed to adjust it's boundaries
	 * automatically.
	 */
	private boolean autoAdjust;
	/**
	 * Specifies whether the viewer is supposed to draw values at the moment.
	 */
	private boolean paused;
	/**
	 * Indicates whether samples have been received yet or not.
	 */
	private boolean firstSample;
	/**
	 * The minimal value to be shown.
	 */
	private double lowerBorderValue;
	/**
	 * The maximal value to be shown.
	 */
	private double upperBorderValue;
	/**
	 * The amount of time to be displayed on the canvas in ms.
	 */
	private long timeToDisplay = 7000L;
	/**
	 * The amount of time that has elapsed already in ms.
	 */
	private long timeElapsed = 0L;
	/**
	 * The amount of time that is equal to the start of the canvas in ms.
	 */
	private long timeAtCanvasStart = 0L;

	// widgets
	private ToolBar optionPanel;
	private InfoBar infoBar;
	private Caption caption;
	private ValueScale vsScale;
	private Canvas canvas;
	private GC canvasGC;

	/**
	 * Creates a new {@link ChartDataViewer}.
	 * 
	 * @param parent
	 *            the {@link Composite} the viewer is to be placed in.
	 * @param displayScale
	 *            determines whether a scale is to be displayed or not.
	 */
	ChartDataViewer(Composite parent, boolean displayScale)
	{
		super(parent, SWT.NONE);
		this.displayScale = displayScale;
		this.displayCaption = false;
		this.autoAdjust = true;
		this.firstSample = true;
		this.lowerBorderValue = -100.0;
		this.upperBorderValue = 100.0;
		this.timeToDisplay = 7000L;
		this.generateWidgets();
	}

	/**
	 * Creates a new {@link ChartDataViewer}.
	 * 
	 * @param parent
	 *            the {@link Composite} the viewer is to be placed in.
	 * @param displayScale
	 *            determines whether a scale is to be displayed or not.
	 * @param autoAdjust
	 *            determines whether the auto-adjust-feature is to be enabled by
	 *            default or not.
	 * @param minValue
	 *            specifies the minimal value to be displayed by default.
	 * @param maxValue
	 *            specifies the maximal value to be displayed by default.
	 * @param timeToDisplay
	 *            specifies the time that is displayed on the x-axis by default.
	 */
	public ChartDataViewer(Composite parent, boolean displayScale, boolean autoAdjust, double minValue,
			double maxValue, long timeToDisplay)
	{
		super(parent, SWT.NONE);
		this.displayScale = displayScale;
		this.displayCaption = false;
		this.autoAdjust = autoAdjust;
		this.firstSample = true;
		this.lowerBorderValue = minValue;
		this.upperBorderValue = maxValue;
		this.timeToDisplay = timeToDisplay;
		this.generateWidgets();
	}

	/**
	 * generates all widgets that are needed.
	 */
	private void generateWidgets()
	{
		this.setLayout(new FormLayout());
		this.setBackgroundMode(SWT.INHERIT_DEFAULT);

		// Options
		this.optionPanel = new ToolBar(this, SWT.NONE);
		FormData fd_optionPanel = new FormData();
		fd_optionPanel.top = new FormAttachment(0);
		fd_optionPanel.left = new FormAttachment(0);
		fd_optionPanel.right = new FormAttachment(100);
		this.optionPanel.setLayoutData(fd_optionPanel);

		DropDownMenu menu = new DropDownMenu(optionPanel, "Options", "Options");

		Option optionPaused = new Option(menu, SWT.CHECK);
		optionPaused.setText("Pause");
		optionPaused.addSelectionListener(new OptionSelectedListener()
		{
			@Override
			public void optionSelected(boolean isSelected)
			{
				ChartDataViewer.this.paused = isSelected;
				if (!isSelected)
				{
					ChartDataViewer.this.resetDataViewer();
				}
			}
		});

		Option optionAutoAdjust = new Option(menu, SWT.CHECK);
		optionAutoAdjust.setText("Auto-Adjust");
		optionAutoAdjust.setSelected(this.autoAdjust);
		optionAutoAdjust.addSelectionListener(new OptionSelectedListener()
		{
			@Override
			public void optionSelected(boolean isSelected)
			{
				ChartDataViewer.this.autoAdjust = isSelected;

				ChartDataViewer.this.vsScale.setEditable(!isSelected);
			}
		});

		Option optionDisplayMarker = new Option(menu, SWT.CHECK);
		optionDisplayMarker.setText("Display Marker On Scale");
		optionDisplayMarker.setSelected(false);
		optionDisplayMarker.addSelectionListener(new OptionSelectedListener()
		{
			@Override
			public void optionSelected(boolean isSelected)
			{
				ChartDataViewer.this.vsScale.displayMarker(isSelected);
			}
		});

		Option optionCaption = new Option(optionPanel, SWT.CHECK);
		optionCaption.setText("Show Caption");
		optionCaption.addSelectionListener(new OptionSelectedListener()
		{
			@Override
			public void optionSelected(boolean isSelected)
			{
				if (ChartDataViewer.this.displayCaption)
				{
					ChartDataViewer.this.displayCaption = false;
					// hide caption
					((FormData) ChartDataViewer.this.caption.getLayoutData()).height = 0;
					ChartDataViewer.this.layout();
				}
				else
				{
					ChartDataViewer.this.displayCaption = true;
					// show caption
					((FormData) ChartDataViewer.this.caption.getLayoutData()).height = 125;
					ChartDataViewer.this.layout();
				}
			}
		});

		// caption
		this.caption = new Caption(this);
		FormData fd_caption = new FormData();
		fd_caption.bottom = new FormAttachment(100);
		fd_caption.left = new FormAttachment(0);
		fd_caption.right = new FormAttachment(100);
		fd_caption.height = 0;
		this.caption.setLayoutData(fd_caption);

		// infobar
		this.infoBar = new InfoBar(this);
		FormData fd_infoBar = new FormData();
		fd_infoBar.bottom = new FormAttachment(caption);
		fd_infoBar.left = new FormAttachment(0);
		fd_infoBar.right = new FormAttachment(100);
		this.infoBar.setLayoutData(fd_infoBar);
		this.infoBar.updateTimeSpanLabel(timeToDisplay);

		// valueLabels
		if (this.displayScale)
		{
			this.vsScale = new ValueScale(this, SWT.VERTICAL, 1, this.lowerBorderValue, this.upperBorderValue);
			FormData fd_vlMax = new FormData();
			fd_vlMax.top = new FormAttachment(this.optionPanel, 0, SWT.BOTTOM);
			fd_vlMax.left = new FormAttachment(0);
			fd_vlMax.width = 75;
			fd_vlMax.bottom = new FormAttachment(this.infoBar);
			this.vsScale.setLayoutData(fd_vlMax);
			this.vsScale.setEditable(!this.autoAdjust);
			this.vsScale.displayMarker(false);
			this.vsScale.addValueListener(new ValueScaleListener()
			{
				@Override
				public void valuesChanged()
				{
					if (!ChartDataViewer.this.autoAdjust)
					{
						double newLowerBorderValue = ChartDataViewer.this.vsScale.getLowerValue();
						double newUpperBorderValue = ChartDataViewer.this.vsScale.getUpperValue();

						if (newLowerBorderValue != ChartDataViewer.this.getLowerBorderValue()
								|| newUpperBorderValue != ChartDataViewer.this.getUpperBorderValue())
						{
							ChartDataViewer.this.adjustBorderValues(newLowerBorderValue, newUpperBorderValue);
						}
					}
				}
			});
		}

		// canvas
		this.canvas = new Canvas(this, SWT.NONE);
		this.canvas.setBackground(this.COLOR_BACKGROUND);
		this.canvas.setLayout(new FormLayout());
		FormData fd_canvas = new FormData();
		if (this.vsScale != null)
		{
			fd_canvas.left = new FormAttachment(this.vsScale, 0, SWT.BOTTOM);
		}
		else
		{
			fd_canvas.left = new FormAttachment(0);
		}
		fd_canvas.top = new FormAttachment(this.optionPanel, 0, SWT.BOTTOM);
		fd_canvas.bottom = new FormAttachment(this.infoBar, 0, SWT.TOP);
		fd_canvas.right = new FormAttachment(100);
		this.canvas.setLayoutData(fd_canvas);
		this.canvasGC = new GC(canvas, SWT.H_SCROLL);
		this.canvas.setCursor(new Cursor(this.getDisplay(), SWT.CURSOR_CROSS));
		// handles mouse-wheel
		this.canvas.addMouseWheelListener(new MouseWheelListener()
		{
			@Override
			public void mouseScrolled(MouseEvent event)
			{
				if (ChartDataViewer.this.timeToDisplay + ((event.count / 3 * -1) * 500) >= 500)
				{
					long timeToDisplay = ChartDataViewer.this.timeToDisplay + (((event.count / 3) * -1) * 500);
					// calculate new timespan
					ChartDataViewer.this.timeToDisplay = timeToDisplay;
					ChartDataViewer.this.infoBar.updateTimeSpanLabel(timeToDisplay);
					ChartDataViewer.this.resetDataViewer();
				}
			}
		});
		this.canvas.addMouseMoveListener(new MouseMoveListener()
		{
			@Override
			public void mouseMove(MouseEvent event)
			{
				if (ChartDataViewer.this.vsScale != null && !ChartDataViewer.this.vsScale.isDisposed())
				{
					ChartDataViewer.this.vsScale.setMarker(event.y);
				}
			}
		});
		this.canvas.addListener(SWT.MouseDown, new Listener()
		{
			@Override
			public void handleEvent(Event event)
			{
				ChartDataViewer.this.canvas.setFocus();
			}
		});
		this.canvas.addListener(SWT.Resize, new Listener()
		{
			@Override
			public void handleEvent(Event event)
			{
				ChartDataViewer.this.resetDataViewer();
			}
		});

		// set backgroundcolor for caption
		this.caption.setCaptionColor(this.canvas.getBackground());
	}

	/**
	 * Resets the {@link DataViewer}.<br />
	 * Will call {@link #reset()}.
	 */
	private void resetDataViewer()
	{
		this.canvasGC.setBackground(this.COLOR_BACKGROUND);
		this.canvasGC.fillRectangle(0, 0, this.canvas.getSize().x, this.canvas.getSize().y);
		this.timeAtCanvasStart = this.timeElapsed;
		this.reset();
	}

	/**
	 * Lets the {@link ChartDataViewer} adjust the minimal and maximal values
	 * that are displayed and redraw it's content according to the new values.
	 * 
	 * @param newLowerBorderValue
	 *            specifies the new minimal value.
	 * @param newUpperBorderValue
	 *            specifies the new maximal value.
	 */
	protected void adjustBorderValues(double newLowerBorderValue, double newUpperBorderValue)
	{
		this.canvasGC.setBackground(this.COLOR_BACKGROUND);
		Point canvasSize = this.canvas.getSize();
		double newSpan = newUpperBorderValue - newLowerBorderValue;

		// calculate upper position of canvas to be set to
		double oldUpperBorderValueTranslatedToNewSpan = Math.abs(newLowerBorderValue) + this.upperBorderValue;
		double oldUpperBorderValuesPercentOfNewSpan = (oldUpperBorderValueTranslatedToNewSpan / (newSpan / 100.0)) / 100.0;
		int oldUpperBorderValueYPos = canvasSize.y - (int) (canvasSize.y * oldUpperBorderValuesPercentOfNewSpan);

		// calculate lower position of canvas to be set to (upper position -
		// lower position = height of canvas to be set to)
		double oldLowerBorderValueTranslatedToNewSpan = Math.abs(newLowerBorderValue) + this.lowerBorderValue;
		double oldLowerBorderValuesPercentOfNewSpan = (oldLowerBorderValueTranslatedToNewSpan / (newSpan / 100.0)) / 100.0;
		int oldLowerBorderValueYPos = canvasSize.y - (int) (canvasSize.y * oldLowerBorderValuesPercentOfNewSpan);

		if (oldLowerBorderValueYPos - oldUpperBorderValueYPos > 0)
		{
			// copy canvas to Image
			Image oldCanvas = new Image(this.getDisplay(), canvasSize.x, canvasSize.y);
			this.canvasGC.copyArea(oldCanvas, 0, 0);
			// reset canvas
			this.canvasGC.fillRectangle(0, 0, canvasSize.x, canvasSize.y);
			// draw Image to new scaled position
			this.canvasGC.drawImage(oldCanvas, 0, 0, canvasSize.x, canvasSize.y, 0, oldUpperBorderValueYPos,
					canvasSize.x, oldLowerBorderValueYPos - oldUpperBorderValueYPos);
		}

		this.lowerBorderValue = newLowerBorderValue;
		this.upperBorderValue = newUpperBorderValue;

		if (this.displayScale && this.vsScale != null && !this.vsScale.isDisposed())
		{
			this.vsScale.setValues(this.lowerBorderValue, this.upperBorderValue);
		}

		this.borderValuesChanged();
	}

	/**
	 * Calculates the positions on canvas for all values given at the time
	 * given.
	 * 
	 * @param values
	 *            the positions are to be calculated for.
	 * @param time
	 *            the positions are to be calculated for.
	 * @return the resulting positions as an array of {@link Point}s.
	 */
	protected Point[] calculatePositionsFor(double[] values, long time)
	{
		long timeRelativeToCanvasStart = time - this.timeAtCanvasStart;
		Point[] positions = new Point[values.length];
		Point canvasSize = this.canvas.getSize();

		double percentOfTimeSpanToDisplay = (timeRelativeToCanvasStart / (this.timeToDisplay / 100.0)) / 100.0;
		int xPos = (int) (canvasSize.x * percentOfTimeSpanToDisplay);

		for (int i = 0; i < values.length; i++)
		{
			double valueSpanToDisplay = this.upperBorderValue - this.lowerBorderValue;
			double valueTranslatedToSpan = Math.abs(this.lowerBorderValue) + values[i];
			double percentOfValueSpanToDisplay = (valueTranslatedToSpan / (valueSpanToDisplay / 100.0)) / 100.0;
			int yPos = canvasSize.y - (int) (canvasSize.y * percentOfValueSpanToDisplay);
			positions[i] = new Point(xPos, yPos);
		}
		return positions;
	}

	/**
	 * Lets the {@link ChartDataViewer} draw data of type
	 * {@link org.xaffect.core.data.DataType}.
	 */
	public void draw(DataType data)
	{
		// create caption if data is first data.
		if (this.firstSample)
		{
			this.fillCaption(data, this.caption);
			this.firstSample = false;
		}

		// update time
		this.timeElapsed = this.getTime(data);
		while (this.timeAtCanvasStart + this.timeToDisplay < this.timeElapsed)
		{
			this.timeAtCanvasStart += this.timeToDisplay;
		}
		this.infoBar.updateTimeLabel(this.timeElapsed);

		if (!this.paused)
		{
			// adjust canvas
			if (this.autoAdjust)
			{
				double[] newBorderValues = this.isDataStillInBorderValues(data);
				if (newBorderValues != null && newBorderValues.length == 2)
				{
					this.adjustBorderValues(newBorderValues[0], newBorderValues[1]);
				}
			}

			// clear area to be clipped
			this.canvasGC.setBackground(this.COLOR_BACKGROUND);
			Rectangle[] clippingAreas = this.calculateClipping(data, this.canvas.getSize());
			for (Rectangle clippingArea : clippingAreas)
			{
				this.canvasGC.fillRectangle(clippingArea);
			}

			// draw values
			this.drawValues(data, this.canvasGC, this.canvas.getSize());
		}
	}

	/**
	 * @return the maximal value that is currently drawable.
	 */
	public double getUpperBorderValue()
	{
		return this.upperBorderValue;
	}

	/**
	 * @return the minimal value that is currently drawable.
	 */
	public double getLowerBorderValue()
	{
		return this.lowerBorderValue;
	}

	/**
	 * @return the current timespan that is displayed on the x-axis.
	 */
	public long getTimeToDisplay()
	{
		return this.timeToDisplay;
	}

	/**
	 * @return the time of the last values that were drawn.
	 */
	public long getTimeElapsed()
	{
		return this.timeElapsed;
	}

	/**
	 * @return the time that is represented at the 0-point of the x-axis.
	 */
	public long getTimeAtCanvasStart()
	{
		return this.timeAtCanvasStart;
	}

	/**
	 * Checks whether given data is still in range of upper and lower
	 * border-values.
	 * 
	 * @param data
	 *            to be checked for.
	 * @return null if data is still in range, double[2]{lowerBorderValue,
	 *         upperBorderValue} if values are to be adjusted.
	 */
	protected abstract double[] isDataStillInBorderValues(DataType data);

	/**
	 * Calculates the time from given data.
	 * 
	 * @param data
	 *            time is to be calculated for.
	 * @return calculated time.
	 */
	protected abstract long getTime(DataType data);

	/**
	 * Calculates the area to clip for given data to be drawn.
	 * 
	 * @param data
	 *            that is to be drawn.
	 * @param canvasSize
	 *            the size of the canvas the data is to be drawn on.
	 * @return an array of {@link Rectangle}s representing the areas to clip.
	 */
	protected abstract Rectangle[] calculateClipping(DataType data, Point canvasSize);

	/**
	 * Draws given values on given {@link GC}.
	 * 
	 * @param data
	 *            to be drawn.
	 * @param canvas
	 *            to be drawn on.
	 * @param canvasSize
	 *            the size of the canvas to be drawn on.
	 */
	protected abstract void drawValues(DataType data, GC canvas, Point canvasSize);

	/**
	 * Handles a change in the minimal or maximal values that are to be drawn.
	 */
	protected abstract void borderValuesChanged();

	/**
	 * Resets the canvas.
	 */
	protected abstract void reset();

	/**
	 * Fills given {@link Caption} with information gathered from given data.
	 * 
	 * @param data
	 *            to collect information from.
	 * @param caption
	 *            to be filled with data.
	 */
	protected abstract void fillCaption(DataType data, Caption caption);
}
