/*
xAffect - Middleware for Physiological Sensors and Affective Signal Processing
Copyright (c) 2011-2012 FZI Research Center for Information Technology, Germany

This file is part of the xAffect middleware. For more information, see http://www.xaffect.org.

This software is licensed under the Original BSD License. You should have received a copy of the Original BSD License along with xAffect. If not, see http://www.xaffect.org/license/.
 */

package org.xaffect.components.sink.dataviewer.resources.chartviewer;

import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Composite;
import org.xaffect.components.sink.dataviewer.resources.common.Formatter;
import org.xaffect.core.data.DataEvent;

/**
 * {@link ChartDataViewer} displaying {@link DataEvent}s.
 */
public class ChartEventViewer extends ChartDataViewer<DataEvent>
{
	private static final int EVENT_RADIUS = 6;
	private static final int CLIPPING_EXTRA = 50;
	private Point lastPosition;
	private DataEvent lastEvent;

	/**
	 * Creates a new {@link ChartEventViewer}.
	 * 
	 * @param parent
	 *            {@link Composite} in which the {@link ChartEventViewer} is
	 *            displayed.
	 */
	public ChartEventViewer(Composite parent)
	{
		super(parent, false);
	}

	@Override
	protected void reset()
	{
		// do nothing
	}

	@Override
	protected double[] isDataStillInBorderValues(DataEvent data)
	{
		// events don't have values -> return null always
		return null;
	}

	@Override
	protected long getTime(DataEvent data)
	{
		return (long) (data.getSamplestamp() * (1000.0 / data.getDescription().getSampleRate()));
	}

	@Override
	protected Rectangle[] calculateClipping(DataEvent data, Point canvasSize)
	{
		Point[] currentPosition = super.calculatePositionsFor(
				new double[] { super.getLowerBorderValue()
						+ (super.getUpperBorderValue() - super.getLowerBorderValue()) / 2 }, this.getTime(data));

		if (this.lastPosition != null && currentPosition != null && currentPosition.length > 0)
		{
			// check whether lastValues are still in same canvas-range
			if (this.getTime(this.lastEvent) > super.getTimeAtCanvasStart())
			{
				// if yes -> clip between the two values
				return new Rectangle[] { new Rectangle(this.lastPosition.x + EVENT_RADIUS / 2, 0, currentPosition[0].x
						- this.lastPosition.x + CLIPPING_EXTRA, canvasSize.y) };
			}
			else
			{
				// if no -> clip from lastValues to canvasEnd and from
				// canvasStart
				// to curentPosition
				return new Rectangle[] {
						new Rectangle(this.lastPosition.x + EVENT_RADIUS / 2, 0, canvasSize.x - this.lastPosition.x,
								canvasSize.y), new Rectangle(0, 0, currentPosition[0].x + CLIPPING_EXTRA, canvasSize.y) };
			}
		}
		else
		{
			return new Rectangle[] {};
		}
	}

	@Override
	protected void drawValues(DataEvent data, GC canvas, Point canvasSize)
	{
		Point[] currentPositions = super.calculatePositionsFor(
				new double[] { super.getLowerBorderValue()
						+ (super.getUpperBorderValue() - super.getLowerBorderValue()) / 2 }, this.getTime(data));

		for (int i = 0; i < currentPositions.length; i++)
		{
			canvas.setBackground(Formatter.getColor(i));
			canvas.fillOval(currentPositions[i].x - EVENT_RADIUS / 2, currentPositions[i].y - EVENT_RADIUS / 2,
					EVENT_RADIUS, EVENT_RADIUS);
		}

		this.lastEvent = data;
		this.lastPosition = currentPositions[0];
	}

	@Override
	protected void borderValuesChanged()
	{
		// events don't have values -> do nothing
		// see isDataStillInBorderValues()
	}

	@Override
	protected void fillCaption(DataEvent data, Caption caption)
	{
		caption.addInfo("Content Class:", data.getDescription().getContentClass());
	}
}
