package org.xaffect.components.sink.dataviewer.resources.chartviewer;

import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Composite;
import org.xaffect.components.sink.dataviewer.resources.common.Formatter;
import org.xaffect.core.data.DataSignal;

/**
 * {@link ChartDataViewer} displaying {@link DataSignal}s.
 */
public class ChartSignalViewer extends ChartDataViewer<DataSignal>
{
	private static final int MIN_CLIPPING_WIDTH = 50;
	private DataSignal lastSignal;
	private Point[] lastPositions;

	/**
	 * Creates a new {@link ChartSignalViewer}.
	 * 
	 * @param parent
	 *            the {@link Composite} the viewer is to be placed in.
	 * @param autoAdjust
	 *            determines whether the auto-adjust-feature is to be enabled by
	 *            default or not.
	 * @param minValue
	 *            specifies the minimal value to be displayed by default.
	 * @param maxValue
	 *            specifies the maximal value to be displayed by default.
	 * @param timeToDisplay
	 *            specifies the timespan to by displayed on the x-axis by
	 *            default.
	 */
	public ChartSignalViewer(Composite parent, boolean autoAdjust, double minValue, double maxValue, long timeToDisplay)
	{
		super(parent, true, autoAdjust, minValue, maxValue, timeToDisplay);
	}

	@Override
	protected long getTime(DataSignal data)
	{
		return (long) (super.getTimeElapsed() + (1000.0 / data.getDescription().getSampleRate()));
	}

	@Override
	protected Rectangle[] calculateClipping(DataSignal data, Point canvasSize)
	{
		if (this.lastPositions != null && this.lastPositions.length > 0)
		{
			// calculate clippingWidth from sampleRate
			double percentageOfCanvas = (((1000.0 / data.getDescription().getSampleRate()) * 50) / (super
					.getTimeToDisplay() / 100.0)) / 100.0;
			int clippingWidth = (int) (percentageOfCanvas * canvasSize.x);

			// check whether clippingWidth is valid
			if (clippingWidth < MIN_CLIPPING_WIDTH)
			{
				clippingWidth = MIN_CLIPPING_WIDTH;
			}

			// check whether clipping exceeds canvas
			if (this.lastPositions[0].x + clippingWidth > canvasSize.x)
			{
				// if yes, clip from lastPositions to end of canvas and to from
				// canvas' beginning until the end of clippingWidth.
				return new Rectangle[] {
						new Rectangle(this.lastPositions[0].x + 1, 0, canvasSize.x - this.lastPositions[0].x,
								canvasSize.y),
						new Rectangle(0, 0, this.lastPositions[0].x + clippingWidth - canvasSize.x, canvasSize.y) };
			}
			else
			{
				// if no, clip from lastPositions until the end of clippingWidth
				return new Rectangle[] { new Rectangle(this.lastPositions[0].x + 1, 0, clippingWidth, canvasSize.y) };
			}
		}
		else
		{
			// no lastPositions -> no clipping
			return new Rectangle[] {};
		}
	}

	@Override
	protected void drawValues(DataSignal data, GC canvas, Point canvasSize)
	{
		// calculate position
		Point[] positions = super.calculatePositionsFor(data.getDataValue(), this.getTime(data));
		if (positions != null && positions.length > 0)
		{
			// check whether positions fit on canvas
			if (positions[0].x > 0 && positions[0].x < canvasSize.x)
			{
				// if yes, draw values
				if (this.lastSignal != null && this.lastPositions != null)
				{
					for (int i = 0; i < this.lastPositions.length && i < positions.length; i++)
					{
						canvas.setForeground(Formatter.getColor(i));
						canvas.drawLine(this.lastPositions[i].x, this.lastPositions[i].y, positions[i].x,
								positions[i].y);
					}
				}
				// store current values for next values
				this.lastSignal = data;
				this.lastPositions = positions;
			}
			else
			{
				// if no, reset
				this.reset();
			}
		}
	}

	@Override
	protected void reset()
	{
		// set lastValues to null to prevent drawValues() to draw a line
		this.lastPositions = null;
		this.lastSignal = null;
	}

	@Override
	protected void borderValuesChanged()
	{
		// update lastPositions
		if (this.lastSignal != null)
		{
			this.lastPositions = super.calculatePositionsFor(this.lastSignal.getDataValue(),
					this.getTime(this.lastSignal));
		}
	}

	@Override
	protected void fillCaption(DataSignal data, Caption caption)
	{
		// update general info
		caption.addInfo("Number of Channels: ", Integer.toString(data.getDescription().getNumberOfChannels()));
		caption.addInfo("Unit: ", data.getDescription().getUnit());
		caption.addInfo("Baseline Value: ", Integer.toString(data.getDescription().getBaseLine()));
		caption.addInfo("LSB Value: ", Double.toString(data.getDescription().getLsbValue()));
		caption.addInfo("ADC Resolution: ", Integer.toString(data.getDescription().getAdcResolution()));
		caption.addInfo("ADC Zero: ", Integer.toString(data.getDescription().getAdcZero()));
		caption.addInfo("Samplerate: ", Double.toString(data.getDescription().getSampleRate()));

		// generate channel-info
		for (int i = 0; i < data.getDescription().getNumberOfChannels(); i++)
		{
			if (data.getDescription().getNamesOfChannels() != null
					&& i < data.getDescription().getNamesOfChannels().length
					&& data.getDescription().getNamesOfChannels()[i] != null
					&& data.getDescription().getNamesOfChannels()[i] != "")
			{
				caption.addChannel(data.getDescription().getNamesOfChannels()[i], Formatter.getColor(i));
			}
			else
			{
				caption.addChannel("[No descriptive name given.]", Formatter.getColor(i));
			}
		}
	}

	@Override
	protected double[] isDataStillInBorderValues(DataSignal data)
	{
		// TODO calculate tolerance
		double newLowerBorderValue = this.getLowerBorderValue();
		double newUpperBorderValue = this.getUpperBorderValue();
		boolean adjustCanvas = false;
		for (double value : data.getDataValue())
		{
			if (value < newLowerBorderValue)
			{
				newLowerBorderValue = value - 10;
				adjustCanvas = true;
			}
			if (value > newUpperBorderValue)
			{
				newUpperBorderValue = value + 10;
				adjustCanvas = true;
			}
		}
		if (adjustCanvas)
		{
			return new double[] { newLowerBorderValue, newUpperBorderValue };
		}
		else
		{
			return null;
		}
	}
}