package org.xaffect.components.sink.dataviewer.resources.chartviewer;

import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Composite;
import org.xaffect.components.sink.dataviewer.resources.common.Formatter;
import org.xaffect.core.data.DataValue;

/**
 * {@link ChartDataViewer} displaying {@link DataValue}s.
 */
public class ChartValueViewer extends ChartDataViewer<DataValue>
{
	private static final int CLIPPING_EXTRA = 50;
	private static final int VALUE_RADIUS = 6;
	private Point[] lastPositions;
	private DataValue lastValue;

	/**
	 * Creates a new {@link ChartValueViewer}.
	 * 
	 * @param parent
	 *            the {@link Composite} the viewer is to be placed in.
	 * @param autoAdjust
	 *            determines whether the auto-adjust-feature is to be enabled by
	 *            default or not.
	 * @param minValue
	 *            specifies the minimal value to be displayed by default.
	 * @param maxValue
	 *            specifies the maximal value to be displayed by default.
	 * @param timeToDisplay
	 *            specifies the time that is displayed on the x-axis by default.
	 */
	public ChartValueViewer(Composite parent, boolean autoAdjust, double minValue, double maxValue, long timeToDisplay)
	{
		super(parent, true, autoAdjust, minValue, maxValue, timeToDisplay);
	}

	@Override
	protected long getTime(DataValue data)
	{
		return (long) (data.getSamplestamp() * (1000.0 / data.getDescription().getSampleRate()));
	}

	@Override
	protected Rectangle[] calculateClipping(DataValue data, Point canvasSize)
	{
		Point[] currentPosition = super.calculatePositionsFor(data.getDataValue(), this.getTime(data));

		if (this.lastPositions != null && this.lastPositions.length > 0 && currentPosition != null
				&& currentPosition.length > 0)
		{
			// check whether lastValues are still in same canvas-range
			if (this.getTime(this.lastValue) > super.getTimeAtCanvasStart())
			{
				// if yes -> clip between the two values
				return new Rectangle[] { new Rectangle(this.lastPositions[0].x + VALUE_RADIUS / 2, 0,
						currentPosition[0].x - this.lastPositions[0].x + CLIPPING_EXTRA, canvasSize.y) };
			}
			else
			{
				// if no -> clip from lastValues to canvasEnd and from
				// canvasStart
				// to curentPosition
				return new Rectangle[] {
						new Rectangle(this.lastPositions[0].x + VALUE_RADIUS / 2, 0, canvasSize.x
								- this.lastPositions[0].x, canvasSize.y),
						new Rectangle(0, 0, currentPosition[0].x + CLIPPING_EXTRA, canvasSize.y) };
			}
		}
		else
		{
			return new Rectangle[] {};
		}
	}

	@Override
	protected void drawValues(DataValue data, GC canvas, Point canvasSize)
	{
		Point[] currentPositions = super.calculatePositionsFor(data.getDataValue(), this.getTime(data));

		if (this.lastValue != null)
		{
			long timeOflastValue = this.getTime(this.lastValue);
			long timeAtCanvasStart = super.getTimeAtCanvasStart();
			// check whether lastValues are still in same canvas-range
			if (timeOflastValue <= timeAtCanvasStart)
			{
				// if no -> recalculate lastPositions
				this.lastPositions = super.calculatePositionsFor(this.lastValue.getDataValue(),
						this.getTime(this.lastValue));
			}

			for (int i = 0; i < this.lastPositions.length && i < currentPositions.length; i++)
			{
				canvas.setForeground(Formatter.getColor(i));
				canvas.drawLine(this.lastPositions[i].x, this.lastPositions[i].y, currentPositions[i].x,
						currentPositions[i].y);
			}
		}

		for (int i = 0; i < currentPositions.length; i++)
		{
			canvas.setBackground(Formatter.getColor(i));
			canvas.fillOval(currentPositions[i].x - VALUE_RADIUS / 2, currentPositions[i].y - VALUE_RADIUS / 2,
					VALUE_RADIUS, VALUE_RADIUS);
		}

		this.lastValue = data;
		this.lastPositions = currentPositions;
	}

	@Override
	protected void borderValuesChanged()
	{
		if (this.lastValue != null)
		{
			// update lastPositions
			this.lastPositions = super.calculatePositionsFor(this.lastValue.getDataValue(),
					this.getTime(this.lastValue));
		}
	}

	@Override
	protected void reset()
	{
		// do nothing
	}

	@Override
	protected void fillCaption(DataValue data, Caption caption)
	{
		// update general info
		caption.addInfo("Number of Channels: ", Integer.toString(data.getDescription().getNumberOfChannels()));
		caption.addInfo("Unit: ", data.getDescription().getUnit());
		caption.addInfo("Baseline Value: ", Integer.toString(data.getDescription().getBaseLine()));
		caption.addInfo("LSB Value: ", Double.toString(data.getDescription().getLsbValue()));
		caption.addInfo("ADC Resolution: ", Integer.toString(data.getDescription().getAdcResolution()));
		caption.addInfo("ADC Zero: ", Integer.toString(data.getDescription().getAdcZero()));

		// generate channel-info
		for (int i = 0; i < data.getDescription().getNumberOfChannels(); i++)
		{
			if (data.getDescription().getNamesOfChannels() != null
					&& i < data.getDescription().getNamesOfChannels().length
					&& data.getDescription().getNamesOfChannels()[i] != null
					&& data.getDescription().getNamesOfChannels()[i] != "")
			{
				caption.addChannel(data.getDescription().getNamesOfChannels()[i], Formatter.getColor(i));
			}
			else
			{
				caption.addChannel("[No descriptive name given.]", Formatter.getColor(i));
			}
		}
	}

	@Override
	protected double[] isDataStillInBorderValues(DataValue data)
	{
		double newLowerBorderValue = this.getLowerBorderValue();
		double newUpperBorderValue = this.getUpperBorderValue();
		boolean adjustCanvas = false;
		for (double value : data.getDataValue())
		{
			if (value < newLowerBorderValue)
			{
				newLowerBorderValue = value - 10;
				adjustCanvas = true;
			}
			if (value > newUpperBorderValue)
			{
				newUpperBorderValue = value + 10;
				adjustCanvas = true;
			}
		}
		if (adjustCanvas)
		{
			return new double[] { newLowerBorderValue, newUpperBorderValue };
		}
		else
		{
			return null;
		}
	}
}