/*
xAffect - Middleware for Physiological Sensors and Affective Signal Processing
Copyright (c) 2011-2012 FZI Research Center for Information Technology, Germany

This file is part of the xAffect middleware. For more information, see http://www.xaffect.org [^]

This software is licensed under the Original BSD License. You should have received a copy of the Original BSD License along with xAffect. If not, see http://www.xaffect.org/license/. [^]
*/ 

package org.xaffect.components.sink.dataviewer.resources.common;

import java.util.Vector;

import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Item;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.ToolBar;
import org.eclipse.swt.widgets.ToolItem;
import org.eclipse.swt.widgets.Widget;

/**
 * Represents an option in a menu. Is able to be placed either as a
 * {@link MenuItem} or a {@link ToolItem}.
 */
public class Option
{
	Widget parent;
	Display display;
	Item representedItem;
	private Vector<OptionSelectedListener> selectedListeners = new Vector<OptionSelectedListener>();

	/**
	 * Creates a new {@link Option} as {@link MenuItem}, so that it can be
	 * placed in a {@link Menu}, respectively {@link DropDownMenu}.
	 * 
	 * @param parent
	 *            {@link Menu} in which the {@link Option} is supposed to be
	 *            placed.
	 * @param style
	 *            SWT-style applicable for {@link MenuItem}
	 */
	public Option(Menu parent, int style)
	{
		this.parent = parent;
		this.display = parent.getDisplay();
		this.representedItem = new MenuItem(parent, style);

		((MenuItem) Option.this.representedItem).addSelectionListener(new SelectionListener()
		{
			@Override
			public void widgetSelected(SelectionEvent arg0)
			{
				for (OptionSelectedListener listener : Option.this.selectedListeners)
				{
					listener.optionSelected(((MenuItem) Option.this.representedItem).getSelection());
				}
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent arg0)
			{
				// nothing for now
			}
		});
	}

	/**
	 * Creates a new {@link Option} as {@link ToolItem}, so that it can be
	 * placed in a {@link ToolBar}.
	 * 
	 * @param parent
	 *            {@link ToolBar} in which the {@link Option} is supposed to be
	 *            placed.
	 * @param style
	 *            SWT-style applicable for {@link ToolItem}
	 */
	public Option(ToolBar parent, int style)
	{
		this.parent = parent;
		this.representedItem = new ToolItem(parent, style);

		((ToolItem) Option.this.representedItem).addSelectionListener(new SelectionListener()
		{
			@Override
			public void widgetSelected(SelectionEvent arg0)
			{
				for (OptionSelectedListener listener : Option.this.selectedListeners)
				{
					listener.optionSelected(((ToolItem) Option.this.representedItem).getSelection());
				}
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent arg0)
			{
				// nothing for now
			}
		});
	}

	/**
	 * Adds given {@link OptionSelectedListener} to {@link Option}'s
	 * listener-pool.
	 * 
	 * @param listener
	 *            {@link OptionSelectedListener} to be added.
	 */
	public void addSelectionListener(OptionSelectedListener listener)
	{
		this.selectedListeners.add(listener);
	}

	/**
	 * Removes given {@link OptionSelectedListener} from {@link Option}'s
	 * listener-pool.
	 * 
	 * @param listener
	 *            {@link OptionSelectedListener} to be removed.
	 */
	public void removeSelectionListener(OptionSelectedListener listener)
	{
		this.selectedListeners.remove(listener);
	}

	/**
	 * Sets text for the {@link Option}
	 * 
	 * @param text
	 *            to be added
	 */
	public void setText(String text)
	{
		this.representedItem.setText(text);
	}

	/**
	 * Returns the {@link Option}'s text.
	 */
	public String getText()
	{
		return representedItem.getText();
	}

	/**
	 * Sets the {@link Option}'s selected-status.
	 */
	public void setSelected(boolean selected)
	{
		if (Option.this.representedItem instanceof ToolItem)
		{
			((ToolItem) Option.this.representedItem).setSelection(selected);
		}
		else if (Option.this.representedItem instanceof MenuItem)
		{
			((MenuItem) Option.this.representedItem).setSelection(selected);
		}
	}

	/**
	 * Returns the {@link Option}'s parent element, which is either a
	 * {@link Menu} or {@link ToolBar}.
	 */
	public Widget getParent()
	{
		return this.parent;
	}

	/**
	 * Sets the {@link Option}'s status (enabled / disabled)
	 * @param enabled whether the {@link Option} is enabled or not.
	 */
	public void setEnabled(final boolean enabled)
	{
		if (Option.this.representedItem instanceof ToolItem)
		{
			((ToolItem) Option.this.representedItem).setEnabled(enabled);
		}
		else if (Option.this.representedItem instanceof MenuItem)
		{
			((MenuItem) Option.this.representedItem).setEnabled(enabled);
		}
	}
}