/*
xAffect - Middleware for Physiological Sensors and Affective Signal Processing
Copyright (c) 2011-2012 FZI Research Center for Information Technology, Germany

This file is part of the xAffect middleware. For more information, see http://www.xaffect.org [^]

This software is licensed under the Original BSD License. You should have received a copy of the Original BSD License along with xAffect. If not, see http://www.xaffect.org/license/. [^]
 */

package org.xaffect.components.sink.dataviewer.resources.common;

import java.util.Vector;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.layout.FormLayout;
import org.eclipse.swt.widgets.Canvas;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.wb.swt.SWTResourceManager;

/**
 * Represents a scale between two values, having two {@link Spinner}s on every
 * end and a {@link Canvas} in between, on which the scale is drawn.<br />
 * <br />
 * Note that although this class is a subclass of Composite, it does not make
 * sense to add children to it, or set a layout on it. <br />
 * <br />
 * <b>Styles:</b>
 * 
 * <pre>
 * HORIZONTAL, VERTICAL
 * </pre>
 * 
 * <b>Restriction:</b>
 * 
 * <pre>
 * This class is <i>not</i> intended to be subclassed.
 * </pre>
 */
public class ValueScale extends Composite
{
	/**
	 * Holds all registered {@link ValueScaleListener}s listening to changes on
	 * values.
	 */
	private Vector<ValueScaleListener> listeners = new Vector<ValueScaleListener>();

	private double minimum = Double.NEGATIVE_INFINITY;
	private double maximum = Double.POSITIVE_INFINITY;
	private double lowerValue;
	private double upperValue;
	private int digits;

	private boolean horizontal;
	private Canvas cvCanvas;
	private GC gcCanvas;
	private Spinner spMaximum;
	private Spinner spMinimum;

	private Image oldMarkerClipping = null;
	private int oldMarkerPosition = 0;
	private boolean displayMarker = true;

	/**
	 * Constructs a new instance of this class given its parent and a style
	 * value describing its appearance.
	 * 
	 * @param parent
	 *            {@link Composite} in which the ValueScale is to be shown.
	 * @param style
	 *            {@link SWT}-Constant describing the ValueScale's appearance
	 *            [defaults to SWT.VERTICAL].
	 * @see {@link ValueScale}
	 */
	public ValueScale(Composite parent, int style, int digits, double lowerValue, double upperValue)
	{
		super(parent, SWT.NO_REDRAW_RESIZE);

		if (style == SWT.HORIZONTAL)
		{
			this.horizontal = true;
		}
		this.digits = digits;
		this.lowerValue = lowerValue;
		this.upperValue = upperValue;

		this.generateWidgets();
	}

	/**
	 * Generates all needed widgets and registers listeners.
	 */
	private void generateWidgets()
	{
		this.setLayout(new FormLayout());

		// create canvas
		this.cvCanvas = new Canvas(this, SWT.NONE);
		FormData fd_cvCanvas = new FormData();
		fd_cvCanvas.top = new FormAttachment(0);
		fd_cvCanvas.left = new FormAttachment(0);
		fd_cvCanvas.right = new FormAttachment(100);
		fd_cvCanvas.bottom = new FormAttachment(100);
		this.cvCanvas.setLayoutData(fd_cvCanvas);
		this.cvCanvas.setLayout(new FormLayout());
		this.gcCanvas = new GC(cvCanvas, SWT.NONE);

		// create Spinners and position them according to the style-attribute
		// received in the constructor.
		this.spMaximum = new Spinner(this.cvCanvas, SWT.NONE);
		FormData fd_spMaximum = new FormData();
		this.spMinimum = new Spinner(this.cvCanvas, SWT.NONE);
		FormData fd_spMinimum = new FormData();
		if (this.horizontal)
		{
			fd_spMinimum.top = new FormAttachment(0, 3);
			fd_spMinimum.left = new FormAttachment(0, 3);
			fd_spMaximum.top = new FormAttachment(0, 3);
			fd_spMaximum.right = new FormAttachment(100, -3);
		}
		else
		{
			fd_spMinimum.bottom = new FormAttachment(100, -3);
			fd_spMinimum.right = new FormAttachment(100, -3);
			fd_spMinimum.left = new FormAttachment(0, 3);
			fd_spMaximum.top = new FormAttachment(0, 3);
			fd_spMaximum.right = new FormAttachment(100, -3);
			fd_spMaximum.left = new FormAttachment(0, 3);
		}
		this.spMaximum.setLayoutData(fd_spMaximum);
		this.spMinimum.setLayoutData(fd_spMinimum);

		// set spinner-info
		this.spMaximum.setDigits(this.digits);
		this.spMinimum.setDigits(this.digits);
		this.spMaximum.setMaximum((int) this.maximum);
		this.spMaximum.setMinimum((int) this.minimum);
		this.spMinimum.setMinimum((int) this.minimum);
		this.spMinimum.setMaximum((int) this.maximum);
		this.spMaximum.setSelection(this.getSpinnerValueFromDouble(this.upperValue));
		this.spMinimum.setSelection(this.getSpinnerValueFromDouble(this.lowerValue));

		// register ModifyListeners to watch upper and lower values.
		this.spMinimum.addModifyListener(new ModifyListener()
		{
			@Override
			public void modifyText(ModifyEvent event)
			{
				try
				{
					double value = Double.parseDouble(ValueScale.this.spMinimum.getText().replace(',', '.'));
					if (ValueScale.this.isVisible() && value != ValueScale.this.lowerValue)
					{
						ValueScale.this.setLowerValue(value);
					}
				}
				catch (NumberFormatException ex)
				{
					// TODO handle NumberFormatException
				}
			}
		});
		this.spMaximum.addModifyListener(new ModifyListener()
		{
			@Override
			public void modifyText(ModifyEvent event)
			{
				try
				{
					double value = Double.parseDouble(ValueScale.this.spMaximum.getText().replace(',', '.'));
					if (ValueScale.this.isVisible() && value != ValueScale.this.upperValue)
					{
						ValueScale.this.setUpperValue(value);
					}
				}
				catch (NumberFormatException ex)
				{
					// TODO handle NumberFormatException
				}
			}
		});

		// register paint-listener to always show the right scale on canvas.
		this.cvCanvas.addPaintListener(new PaintListener()
		{
			@Override
			public void paintControl(PaintEvent event)
			{
				ValueScale.this.redraw();
			}
		});
	}

	/**
	 * Calculates and draws the right scale between the two border-values.
	 */
	@Override
	public void redraw()
	{
		super.layout();
		super.redraw();
		Point canvasSize = this.cvCanvas.getSize();
		this.gcCanvas.setBackground(SWTResourceManager.getColor(SWT.COLOR_GRAY));
		this.gcCanvas.fillRectangle(0, 0, canvasSize.x, canvasSize.y);
		// redraw canvas
		if (this.horizontal)
		{
			drawScaleHorizontally(0, this.lowerValue, canvasSize.x, this.upperValue, 1);
		}
		else
		{
			drawScaleVertically(0, this.upperValue, canvasSize.y, this.lowerValue, 1);
		}
	}

	/**
	 * Draws the scale between the start-value at the start-position and the
	 * end-value at the end-position recursively, in a vertical direction.<br />
	 * callNumber specifies how often the function is called in the recursive
	 * stack. Depending on that parameter, the function will draw the lines
	 * representing values in different sizes.
	 * 
	 * @param startYPos
	 *            the y-position on the canvas to begin drawing at.
	 * @param startValue
	 *            the value to be represented by startYPos.
	 * @param endYPos
	 *            the y-position on the canvas to end drawing at.
	 * @param endValue
	 *            the value to be represented by endYPos.
	 * @param callNumber
	 *            Representing the call-counter in the recursive call-stack.
	 *            [Initiate with 1]
	 */
	private void drawScaleVertically(int startYPos, double startValue, int endYPos, double endValue, int callNumber)
	{
		// draw value in the middle
		Point canvasSize = this.cvCanvas.getSize();
		double midValue = startValue + (endValue - startValue) / 2;
		int midYPos = startYPos + (endYPos - startYPos) / 2;
		int lineWidth = 25 - callNumber * 3;

		if (midYPos > this.spMaximum.getSize().y + 5 && midYPos < canvasSize.y - this.spMinimum.getSize().y - 5)
		{
			this.gcCanvas.drawLine(canvasSize.x - lineWidth, midYPos, canvasSize.x, midYPos);
		}

		if (callNumber * 30 < (canvasSize.y / callNumber) && midYPos > this.spMaximum.getSize().y + 20
				&& midYPos < canvasSize.y - this.spMinimum.getSize().y - 20)
		{
			// draw string
			String floorValue = Double.toString(Math.floor(midValue * Math.pow(10, this.spMaximum.getDigits() + 1))
					/ Math.pow(10, this.spMaximum.getDigits() + 1));
			this.gcCanvas.drawString(floorValue, canvasSize.x - lineWidth - (floorValue.length() * 5) - 5, midYPos - 7);
		}

		// draw further recursively
		if ((endYPos - startYPos) / 2 > 5)
		{
			drawScaleVertically(startYPos, startValue, midYPos, midValue, callNumber + 1);
			drawScaleVertically(midYPos, midValue, endYPos, endValue, callNumber + 1);
		}
	}

	/**
	 * Draws the scale between the start-value at the start-position and the
	 * end-value at the end-position recursively, in a horizontal direction.<br />
	 * callNumber specifies how often the function is called in the recursive
	 * stack. Depending on that parameter, the function will draw the lines
	 * representing values in different sizes.
	 * 
	 * @param startXPos
	 *            the x-position on the canvas to begin drawing at.
	 * @param startValue
	 *            the value to be represented by startYPos.
	 * @param endXPos
	 *            the x-position on the canvas to end drawing at.
	 * @param endValue
	 *            the value to be represented by endYPos.
	 * @param callNumber
	 *            Representing the call-counter in the recursive call-stack.
	 *            [Initiate with 1]
	 */
	private void drawScaleHorizontally(int startXPos, double startValue, int endXPos, double endValue, int callNumber)
	{
		// draw value in the middle
		Point canvasSize = this.cvCanvas.getSize();
		double midValue = startValue + (endValue - startValue) / 2;
		int midXPos = startXPos + (endXPos - startXPos) / 2;
		int lineHeight = (25 - callNumber * 3);

		if (midXPos > this.spMinimum.getSize().x + 5 && midXPos < canvasSize.x - this.spMaximum.getSize().x - 5)
		{
			this.gcCanvas.drawLine(midXPos, lineHeight, midXPos, 0);
		}

		if (callNumber * 50 < (canvasSize.x / callNumber) && midXPos > this.spMinimum.getSize().x + 20
				&& midXPos < canvasSize.x - this.spMaximum.getSize().x - 20)
		{
			// draw string
			String floorValue = Double.toString(Math.floor(midValue * Math.pow(10, this.spMaximum.getDigits() + 1))
					/ Math.pow(10, this.spMaximum.getDigits() + 1));
			this.gcCanvas.drawString(floorValue, midXPos - (floorValue.length() * 5) / 2, lineHeight + 5);
		}

		// draw further recursively
		if ((endXPos - startXPos) / 2 > 5)
		{
			drawScaleHorizontally(startXPos, startValue, midXPos, midValue, callNumber + 1);
			drawScaleHorizontally(midXPos, midValue, endXPos, endValue, callNumber + 1);
		}
	}

	/**
	 * Sets the position of a marker to the {@link ValueScale}, displayed by a red line.
	 * @param pos Position of the marker to be set.
	 */
	public void setMarker(int pos)
	{
		if (this.displayMarker && pos != this.oldMarkerPosition)
		{
			if (this.horizontal)
			{
				if (this.oldMarkerClipping != null)
				{
					// restore old image
					this.gcCanvas.drawImage(this.oldMarkerClipping, this.oldMarkerPosition, 0);
				}
				// store current image
				this.oldMarkerClipping = new Image(this.getDisplay(), 1, this.cvCanvas.getSize().y);
				this.gcCanvas.copyArea(this.oldMarkerClipping, pos, 0);
				this.oldMarkerPosition = pos;
				// draw line
				this.gcCanvas.setForeground(SWTResourceManager.getColor(SWT.COLOR_RED));
				this.gcCanvas.drawLine(pos, 0, pos, this.cvCanvas.getSize().y);
				this.gcCanvas.setForeground(SWTResourceManager.getColor(SWT.COLOR_BLACK));
			}
			else
			{
				if (this.oldMarkerClipping != null)
				{
					// restore old image
					this.gcCanvas.drawImage(this.oldMarkerClipping, 0, this.oldMarkerPosition);
				}
				// store current image
				this.oldMarkerClipping = new Image(this.getDisplay(), this.cvCanvas.getSize().x, 1);
				this.gcCanvas.copyArea(this.oldMarkerClipping, 0, pos);
				this.oldMarkerPosition = pos;
				// draw line
				this.gcCanvas.setForeground(SWTResourceManager.getColor(SWT.COLOR_RED));
				this.gcCanvas.drawLine(0, pos, this.cvCanvas.getSize().x, pos);
				this.gcCanvas.setForeground(SWTResourceManager.getColor(SWT.COLOR_BLACK));
			}
		}
	}
	
	/**
	 * Determines whether the marker is to be displayed or not.
	 * @param displayMarker whether the marker is to be displayed or not.
	 */
	public void displayMarker(boolean displayMarker)
	{
		if(!displayMarker && this.isVisible())
		{
			this.setMarker(-1);
		}
		this.displayMarker = displayMarker;
	}

	/**
	 * Sets the minimal and maximal values the {@link ValueScale} is able to
	 * display.
	 * 
	 * @param minimum
	 *            minimal value the ValueScale will represent.
	 * @param maximum
	 *            maximal value the ValueScale will represent.
	 */
	public void setMaxima(double minimum, double maximum)
	{
		// set internal values
		this.minimum = minimum;
		this.maximum = maximum;
	}

	/**
	 * @return the minimal value the {@link ValueScale} will represent.
	 */
	public double getMinimum()
	{
		return this.minimum;
	}

	/**
	 * @return the maximal value the {@link ValueScale} will represent.
	 */
	public double getMaximum()
	{
		return this.maximum;
	}

	/**
	 * Sets both selected values at once.
	 * 
	 * @param lowerValue
	 *            the lower border-value the {@link ValueScale} is to be
	 *            representing.
	 * @param upperValue
	 *            the upper border-value the ValueScale is to be representing.
	 */
	public void setValues(double lowerValue, double upperValue)
	{
		this.setLowerValue(lowerValue);
		this.setUpperValue(upperValue);
	}

	/**
	 * Sets the lower border-value the {@link ValueScale} is to be representing.
	 * 
	 * @param lowerValue
	 *            value to be set.
	 */
	public void setLowerValue(double lowerValue)
	{
		if (lowerValue >= this.minimum && lowerValue < this.upperValue)
		{
			this.lowerValue = lowerValue;
			this.spMinimum.setSelection(this.getSpinnerValueFromDouble(lowerValue));
			this.notifyListeners();
			this.redraw();
		}
		else
		{
			this.spMinimum.setSelection(this.getSpinnerValueFromDouble(this.lowerValue));
		}
	}

	/**
	 * @return the currently selected lower border-value.
	 */
	public double getLowerValue()
	{
		return this.lowerValue;
	}

	/**
	 * Sets the upper border-value the {@link ValueScale} is to be representing.
	 * 
	 * @param upperValue
	 *            value to be set.
	 */
	public void setUpperValue(double upperValue)
	{
		if (upperValue <= this.maximum && upperValue > this.lowerValue)
		{
			this.upperValue = upperValue;
			this.spMaximum.setSelection(this.getSpinnerValueFromDouble(upperValue));
			this.notifyListeners();
			this.redraw();
		}
		else
		{
			this.spMaximum.setSelection(this.getSpinnerValueFromDouble(this.upperValue));
		}
	}

	/**
	 * @return the currently selected upper border-value.
	 */
	public double getUpperValue()
	{
		return this.upperValue;
	}

	/**
	 * Determines whether the border-values are to be editable by the user or
	 * not.
	 * 
	 * @param editable
	 *            whether the GUI is to be editable or not.
	 */
	public void setEditable(boolean editable)
	{
		this.spMaximum.setEnabled(editable);
		this.spMinimum.setEnabled(editable);
	}

	/**
	 * @return whether the border-values are to be editable by the user or not.
	 */
	public boolean getEditable()
	{
		return this.spMaximum.getEnabled();
	}

	/**
	 * Sets the value-{@link Spinner}s' digits after comma.
	 * 
	 * @param digits
	 *            number of digits to be set.
	 */
	public void setDigits(int digits)
	{
		this.digits = digits;
		this.spMaximum.setDigits(digits);
		this.spMinimum.setDigits(digits);
	}

	/**
	 * @return the value-{@link Spinner}s' digits after comma.
	 */
	public int getDigits()
	{
		return this.digits;
	}

	/**
	 * Registers a {@link ValueScaleListener} to listen to changes made to
	 * selected values.
	 * 
	 * @param listener
	 *            to be registered.
	 */
	public void addValueListener(ValueScaleListener listener)
	{
		this.listeners.add(listener);
	}

	/**
	 * Unregisters a {@link ValueScaleListener}.
	 * 
	 * @param listener
	 *            to be unregistered.
	 */
	public void removeValueListener(ModifyListener listener)
	{
		this.listeners.remove(listener);
	}

	/**
	 * Notifies all registered {@link ValueScaleListener}s that changes have
	 * benn made to the selected values.
	 */
	private void notifyListeners()
	{
		for (ValueScaleListener listener : this.listeners)
		{
			listener.valuesChanged();
		}
	}

	/**
	 * Calculates the value to be used for Spinner.setSelection() from double.
	 * 
	 * @param value
	 *            to be converted.
	 * @return Converted value.
	 */
	private int getSpinnerValueFromDouble(double value)
	{
		return (int) (value * Math.pow(10, this.digits));
	}
}
