/*
xAffect - Middleware for Physiological Sensors and Affective Signal Processing
Copyright (c) 2011-2012 FZI Research Center for Information Technology, Germany

This file is part of the xAffect middleware. For more information, see http://www.xaffect.org.

This software is licensed under the Original BSD License. You should have received a copy of the Original BSD License along with xAffect. If not, see http://www.xaffect.org/license/.
 */
package org.xaffect.control.gui;

import java.util.HashMap;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.layout.FormLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Text;
import org.xaffect.setup.Setup;

/**
 * Contains a {@link Combo}-field containing all setups that are available in
 * the current xAffect-installation, as well as a Composite containing all
 * required parameters for the currently selected {@link Setup}.<br />
 * The {@link Composite} will fetch all needed data by itself and does not need
 * to be configured.
 * 
 * @author platzek
 */
public class CompositeConfiguration extends Composite
{
	// widgets
	private Group grpConfiguration;
	private Combo comboSelectSetup;
	private Composite cmpSetupParameters;
	private Label[] lblSetupParameterKeys;
	private Control[] ctrlSetupParameterValues;
	private Button btnConfigure;

	// data
	private String[] setups;
	private String[] setupParameters;
	private Setup setup;

	/**
	 * Creates a new {@link CompositeConfiguration}.
	 * 
	 * @see Composite
	 * @param parent
	 *            see {@link Composite}
	 * @param style
	 *            see {@link Composite}
	 */
	public CompositeConfiguration(Composite parent, int style, String[] setups)
	{
		super(parent, style);

		this.setups = setups;
		this.generateWidgets();
		this.updateParameters();
	}

	/**
	 * Generates all widgets needed by the {@link CompositeConfiguration}.
	 */
	private void generateWidgets()
	{
		this.setLayout(new FormLayout());

		this.lblSetupParameterKeys = new Label[0];
		this.ctrlSetupParameterValues = new Text[0];

		this.grpConfiguration = new Group(this, SWT.NONE);
		this.grpConfiguration.setText("Configuration");
		this.grpConfiguration.setLayout(new FormLayout());
		FormData fd_grpConfiguration = new FormData();
		fd_grpConfiguration.bottom = new FormAttachment(100);
		fd_grpConfiguration.top = new FormAttachment(0);
		fd_grpConfiguration.left = new FormAttachment(0);
		fd_grpConfiguration.right = new FormAttachment(100);
		this.grpConfiguration.setLayoutData(fd_grpConfiguration);

		Label lblSelectSetup = new Label(this.grpConfiguration, SWT.NONE);
		FormData fd_lblSelectSetup = new FormData();
		fd_lblSelectSetup.top = new FormAttachment(0, 10);
		fd_lblSelectSetup.left = new FormAttachment(0, 5);
		lblSelectSetup.setLayoutData(fd_lblSelectSetup);
		lblSelectSetup.setText("Select Setup");

		this.comboSelectSetup = new Combo(this.grpConfiguration, SWT.READ_ONLY);
		FormData fd_comboSelectSetup = new FormData();
		fd_comboSelectSetup.top = new FormAttachment(lblSelectSetup, 5);
		fd_comboSelectSetup.left = new FormAttachment(lblSelectSetup, 0, SWT.LEFT);
		fd_comboSelectSetup.right = new FormAttachment(100, -5);
		this.comboSelectSetup.setLayoutData(fd_comboSelectSetup);
		this.comboSelectSetup.setItems(this.setups);
		this.comboSelectSetup.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				updateParameters();
			}
		});
		
		if(this.comboSelectSetup.getItemCount() > 0)
		{
			this.comboSelectSetup.select(0);
		}

		this.btnConfigure = new Button(this.grpConfiguration, SWT.NONE);
		FormData fd_btnConnect = new FormData();
		fd_btnConnect.right = new FormAttachment(100, -5);
		fd_btnConnect.bottom = new FormAttachment(100, -5);
		this.btnConfigure.setLayoutData(fd_btnConnect);
		this.btnConfigure.setText("Configure");

		this.cmpSetupParameters = new Composite(this.grpConfiguration, SWT.V_SCROLL);
		this.cmpSetupParameters.setLayout(new GridLayout(2, true));
		FormData fd_composite = new FormData();
		fd_composite.top = new FormAttachment(this.comboSelectSetup, 5);
		fd_composite.left = new FormAttachment(lblSelectSetup, 0, SWT.LEFT);
		fd_composite.right = new FormAttachment(this.comboSelectSetup, 0, SWT.RIGHT);
		fd_composite.bottom = new FormAttachment(this.btnConfigure, -5, SWT.TOP);
		this.cmpSetupParameters.setLayoutData(fd_composite);
	}

	/**
	 * Updates the {@link Composite} in which the parameters for the currently
	 * selected {@link Setup} are displayed.<br />
	 * Is usually called as soon as the currently selected setup changes.
	 */
	private void updateParameters()
	{
		// clear old data and widgets
		for (Control control : this.cmpSetupParameters.getChildren())
		{
			control.dispose();
		}
		this.lblSetupParameterKeys = null;
		this.ctrlSetupParameterValues = null;
		this.setup = null;
		this.setupParameters = null;

		// get new parameters
		try
		{
			String setup = this.comboSelectSetup.getText();
			if (setup != null && setup != "")
			{
				this.setup = (Setup) XAffectWindowControl.class.getClassLoader().loadClass(setup).newInstance();
				this.setupParameters = this.setup.getKeys();

				if (this.setupParameters != null)
				{
					// create parameters as widgets
					this.lblSetupParameterKeys = new Label[this.setupParameters.length];
					this.ctrlSetupParameterValues = new Control[this.setupParameters.length];

					if (this.cmpSetupParameters != null && !this.cmpSetupParameters.isDisposed())
					{
						for (int i = 0; i < this.setupParameters.length; i++)
						{
							// create label
							Label lblKey = new Label(this.cmpSetupParameters, SWT.NONE);
							lblKey.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
							lblKey.setText(this.setupParameters[i].replace("CHECKBOX_", "") + ":");

							// create value-control
							Control value;

							// if parameter contains "checkbox_" create checkbox
							// as value
							if (this.setupParameters[i].contains("CHECKBOX_"))
							{
								value = new Button(this.cmpSetupParameters, SWT.CHECK);
								value.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
							}
							// else create text as value
							else
							{
								value = new Text(this.cmpSetupParameters, SWT.BORDER);
								value.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
							}

							this.lblSetupParameterKeys[i] = lblKey;
							this.ctrlSetupParameterValues[i] = value;
						}
						this.cmpSetupParameters.layout();
					}
				}
			}

			// enable configure-button
			if (this.btnConfigure != null && this.btnConfigure.isDisposed())
			{
				this.btnConfigure.setEnabled(true);
			}
		}
		catch (Exception e)
		{
			// disable configure-button and log error
			if (this.btnConfigure != null && this.btnConfigure.isDisposed())
			{
				this.btnConfigure.setEnabled(false);
			}
		}
	}

	/**
	 * Registers a {@link Listener} which will be notified as soon as the
	 * configure-button is pressed.
	 * 
	 * @param listener
	 *            to be added.
	 */
	public void addConfigureListener(Listener listener)
	{
		this.btnConfigure.addListener(SWT.Selection, listener);
	}

	/**
	 * @return the currently selected {@link Setup}.
	 */
	public Setup getSetup()
	{
		return this.setup;
	}

	/**
	 * @return a {@link HashMap}<String, String>, which contains all required
	 *         parameters for the currently selected {@link Setup}.
	 */
	public HashMap<String, String> getParameters()
	{
		HashMap<String, String> parameters = new HashMap<String, String>();

		if (this.setupParameters != null)
		{
			for (int i = 0; i < this.setupParameters.length; i++)
			{
				// receive parameters from widgets and add them to HashMap
				if (this.ctrlSetupParameterValues[i] instanceof Button)
				{
					boolean selection = ((Button) this.ctrlSetupParameterValues[i]).getSelection();
					String txtSelection = new Boolean(selection).toString();
					parameters.put(this.setupParameters[i], txtSelection);
				}
				else if (this.ctrlSetupParameterValues[i] instanceof Text)
				{
					parameters.put(this.setupParameters[i], ((Text) this.ctrlSetupParameterValues[i]).getText());
				}
			}
		}

		return parameters;
	}

	@Override
	public void setEnabled(boolean enabled)
	{
		if (this.comboSelectSetup != null && !this.comboSelectSetup.isDisposed() && this.cmpSetupParameters != null
				&& !this.cmpSetupParameters.isDisposed() && this.btnConfigure != null
				&& !this.btnConfigure.isDisposed())
		{
			this.comboSelectSetup.setEnabled(enabled);
			for (Control widget : this.cmpSetupParameters.getChildren())
			{
				if (widget != null && !widget.isDisposed())
				{
					widget.setEnabled(enabled);
				}
			}
			this.btnConfigure.setEnabled(enabled);
		}
	}
}