/*
xAffect - Middleware for Physiological Sensors and Affective Signal Processing
Copyright (c) 2011-2012 FZI Research Center for Information Technology, Germany

This file is part of the xAffect middleware. For more information, see http://www.xaffect.org.

This software is licensed under the Original BSD License. You should have received a copy of the Original BSD License along with xAffect. If not, see http://www.xaffect.org/license/.
 */
package org.xaffect.control.gui;

import java.util.Timer;
import java.util.TimerTask;
import java.util.Vector;

/**
 * Used to time the duration of any measurement. Will trigger all registered
 * {@link MeasurementTimeListener} with a frequency of 1sec and pass a
 * well-formatted String resembling the passed time.
 */
public class MeasurementTimer
{
	private Timer timer;
	private TimerTask task;
	private long startTime;
	private Vector<MeasurementTimeListener> listeners;

	/**
	 * Creates a new {@link MeasurementTimer}.
	 * 
	 * @param state
	 */
	public MeasurementTimer()
	{
		this.timer = new Timer();
		this.listeners = new Vector<MeasurementTimeListener>();
	}

	/**
	 * Starts the {@link MeasurementTimer}.
	 */
	public void start()
	{
		this.startTime = System.currentTimeMillis();
		this.task = new TimerTask()
		{
			public void run()
			{
				updateListeners();
			}
		};
		// run task every second
		this.timer.scheduleAtFixedRate(this.task, 1000, 1000);
	}

	/**
	 * Stops the {@link MeasurementTimer}.
	 */
	public void stop()
	{
		if (this.timer != null)
		{
			this.timer.cancel();
		}
	}

	/**
	 * Registers a {@link MeasurementTimeListener} to the
	 * {@link MeasurementTimer}. The Listener will be triggered every second and
	 * passed a well-formatted String resembling the passed time.
	 * 
	 * @param listener to be registered.
	 */
	public void addListener(MeasurementTimeListener listener)
	{
		this.listeners.add(listener);
	}

	/**
	 * Notifies every listener and passes the current time.
	 */
	private void updateListeners()
	{
		long elapsedTime = System.currentTimeMillis() - this.startTime;
		int seconds = (int) (elapsedTime / 1000) % 60;
		int minutes = (int) ((elapsedTime / (1000 * 60)) % 60);
		int hours = (int) ((elapsedTime / (1000 * 60 * 60)) % 24);

		String time = String.format("%02d", hours) + ":" + String.format("%02d", minutes) + ":"
				+ String.format("%02d", seconds);

		for (MeasurementTimeListener listener : this.listeners)
		{
			listener.timeChanged(time);
		}
	}
}
