/*
xAffect - Middleware for Physiological Sensors and Affective Signal Processing
Copyright (c) 2011-2012 FZI Research Center for Information Technology, Germany

This file is part of the xAffect middleware. For more information, see http://www.xaffect.org.

This software is licensed under the Original BSD License. You should have received a copy of the Original BSD License along with xAffect. If not, see http://www.xaffect.org/license/.
 */
package org.xaffect.control.gui;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Calendar;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.layout.FormLayout;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.ProgressBar;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.xaffect.config.ConstantsDesktop;
import org.xaffect.control.Control;
import org.xaffect.core.LifeCycleManager;
import org.xaffect.core.common.State;

/**
 * The Window class is an implementation of the control interface with a
 * graphical user interface (GUI). You can choose this control by editing the
 * CONTROL_INTERFACE parameter in the xaffect.properties file.
 * 
 * Use the GUI for xAffect as stand alone application or when xAffect is the
 * master application in your measurement framework.
 * 
 * @author platzek
 * 
 */
public class XAffectWindowControl extends Control
{
	private Display display;
	private Shell shell;

	// widgets
	private CompositeSubjectInfo subjectInfo;
	private CompositeConfiguration configurationInfo;
	private CompositeControl controlInfo;
	private ProgressBar progressBar;
	private Text textLogging;

	// data
	private String[] setups;
	private String[] availableStudyIds;
	private String[] availableInstitutionIds;

	/**
	 * Creates a new {@link XAffectWindowControl}
	 */
	public XAffectWindowControl(LifeCycleManager manager, String[] setups, String[] availableStudyIds,
			String[] availableInstitutionIds)
	{
		super(manager);

		// set up
		this.display = Display.getDefault();

		this.shell = new Shell(display, SWT.SHELL_TRIM);
		this.shell.setText("xAffect");
		this.shell.setSize(450, 600);
		this.shell.setImage(new Image(display, ConstantsDesktop.IMAGE_PATH + ConstantsDesktop.IMAGE_ICON));

		this.setups = setups;
		this.availableInstitutionIds = availableInstitutionIds;
		this.availableStudyIds = availableStudyIds;

		this.generateWidgets();
	}

	/**
	 * Generates all needed widgets.
	 */
	private void generateWidgets()
	{
		// set layout
		FormLayout formLayout = new FormLayout();
		formLayout.marginTop = 5;
		formLayout.marginRight = 5;
		formLayout.marginLeft = 5;
		formLayout.marginHeight = 5;
		formLayout.marginBottom = 5;
		this.shell.setLayout(formLayout);

		this.subjectInfo = new CompositeSubjectInfo(this.shell, SWT.NONE, this.availableStudyIds,
				this.availableInstitutionIds);
		FormData fd_subjectInfo = new FormData();
		fd_subjectInfo.top = new FormAttachment(0);
		fd_subjectInfo.left = new FormAttachment(0);
		fd_subjectInfo.right = new FormAttachment(100);
		this.subjectInfo.setLayoutData(fd_subjectInfo);

		this.configurationInfo = new CompositeConfiguration(this.shell, SWT.NONE, this.setups);
		FormData fd_configurationInfo = new FormData();
		fd_configurationInfo.top = new FormAttachment(this.subjectInfo, 5);
		fd_configurationInfo.left = new FormAttachment(0);
		fd_configurationInfo.right = new FormAttachment(100);
		fd_configurationInfo.bottom = new FormAttachment(66);
		this.configurationInfo.setLayoutData(fd_configurationInfo);

		this.controlInfo = new CompositeControl(this.shell, SWT.NONE);
		FormData fd_controlInfo = new FormData();
		fd_controlInfo.top = new FormAttachment(this.configurationInfo, 5);
		fd_controlInfo.left = new FormAttachment(0);
		fd_controlInfo.right = new FormAttachment(100);
		this.controlInfo.setLayoutData(fd_controlInfo);

		this.progressBar = new ProgressBar(this.shell, SWT.NONE);
		FormData fd_progressBar = new FormData();
		fd_progressBar.left = new FormAttachment(0);
		fd_progressBar.right = new FormAttachment(100);
		fd_progressBar.bottom = new FormAttachment(100);
		this.progressBar.setLayoutData(fd_progressBar);
		this.progressBar.setVisible(false);

		this.textLogging = new Text(this.shell, SWT.BORDER | SWT.READ_ONLY | SWT.V_SCROLL);
		FormData fd_textLogging = new FormData();
		fd_textLogging.top = new FormAttachment(this.controlInfo, 5);
		fd_textLogging.left = new FormAttachment(0);
		fd_textLogging.right = new FormAttachment(100);
		fd_textLogging.bottom = new FormAttachment(this.progressBar, -5);
		this.textLogging.setLayoutData(fd_textLogging);

		this.addListeners();
	}

	/**
	 * Registers listeners to widgets and {@link LifeCycleManager} to react to
	 * interaction and {@link State}.
	 */
	private void addListeners()
	{
		// watch btnConfigure and configure
		this.configurationInfo.addConfigureListener(new Listener()
		{
			public void handleEvent(Event e)
			{
				XAffectWindowControl.this.configure();
				XAffectWindowControl.this.updateProgressbar(true, 0);
			}
		});

		// watch btnStartTraining and start training
		this.controlInfo.addTrainingListener(new Listener()
		{
			public void handleEvent(Event e)
			{
				// start training
				XAffectWindowControl.super.getLifeCycleManager().train();
				// log and disable button
				XAffectWindowControl.this.log("Started Training.");
			}
		});

		// watch btnStartMeasuring and start measuring
		this.controlInfo.addMeasuringListener(new Listener()
		{
			public void handleEvent(Event e)
			{
				XAffectWindowControl.super.getLifeCycleManager().measure();
				XAffectWindowControl.this.log("Started Measuring.");
			}
		});

		// watch btnStop and stop measuring
		this.controlInfo.addStopListener(new Listener()
		{
			public void handleEvent(Event e)
			{
				XAffectWindowControl.super.getLifeCycleManager().stop();
				XAffectWindowControl.this.log("Stopped Measuring.");
			}
		});

		// watch btnNextSubject and change restart with next subject
		this.controlInfo.addNextSubjectListener(new Listener()
		{
			public void handleEvent(Event e)
			{
				XAffectWindowControl.super.getLifeCycleManager().restart();
				XAffectWindowControl.this.subjectInfo.setEnabled(true);
				XAffectWindowControl.this.configurationInfo.setEnabled(true);
				XAffectWindowControl.this.controlInfo.setEnabled(false);
			}
		});

		// watch status-updates and disable/enable widgets adequately
		super.getLifeCycleManager().registerStatusChangedListener(new PropertyChangeListener()
		{
			@Override
			public void propertyChange(PropertyChangeEvent evt)
			{
				switch ((State) evt.getNewValue())
				{
					case START:
						XAffectWindowControl.this.display.syncExec(new Runnable()
						{
							@Override
							public void run()
							{
								// update subject-info-group
								if (XAffectWindowControl.this.subjectInfo != null
										&& !XAffectWindowControl.this.subjectInfo.isDisposed())
								{
									XAffectWindowControl.this.subjectInfo.setEnabled(false);
								}

								// update configuration-group
								if (XAffectWindowControl.this.configurationInfo != null
										&& !XAffectWindowControl.this.configurationInfo.isDisposed())
								{
									XAffectWindowControl.this.configurationInfo.setEnabled(false);
								}

								// update control-group
								if (XAffectWindowControl.this.controlInfo != null
										&& !XAffectWindowControl.this.controlInfo.isDisposed())
								{
									XAffectWindowControl.this.controlInfo.setEnabled(false);
								}
							}
						});
						break;
					case INIT:
						// does not affect UI
						break;
					case CONFIG:
						XAffectWindowControl.this.updateProgressbar(false, 0);
						XAffectWindowControl.this.display.syncExec(new Runnable()
						{
							@Override
							public void run()
							{
								// update control-group
								if (XAffectWindowControl.this.controlInfo != null
										&& !XAffectWindowControl.this.controlInfo.isDisposed())
								{
									XAffectWindowControl.this.controlInfo.enableTraining();
								}
							}
						});
						break;
					case TRAINING:
						XAffectWindowControl.this.display.syncExec(new Runnable()
						{
							@Override
							public void run()
							{
								// update control-group
								if (XAffectWindowControl.this.controlInfo != null
										&& !XAffectWindowControl.this.controlInfo.isDisposed())
								{
									XAffectWindowControl.this.controlInfo.enableStopping();

									// enable measuring-button if possible, will
									// enable itself as soon as possible
									// otherwise
									if (!XAffectWindowControl.super.getLifeCycleManager().stillRequiresTraining())
									{
										XAffectWindowControl.this.controlInfo.enableMeasuring();
									}
								}
							}
						});
						break;
					case MEASURING:
						// no changes to be made, since measuring-button
						// will disable itself
						break;
					case STOPPED:
						XAffectWindowControl.this.display.syncExec(new Runnable()
						{
							@Override
							public void run()
							{
								// update control-group
								if (XAffectWindowControl.this.controlInfo != null
										&& !XAffectWindowControl.this.controlInfo.isDisposed())
								{
									// stop-button will disable itself
									XAffectWindowControl.this.controlInfo.enableNextSubject();
								}
							}
						});
						break;
					case UNKNOWN:
						XAffectWindowControl.this.display.syncExec(new Runnable()
						{
							@Override
							public void run()
							{
								if (XAffectWindowControl.this.controlInfo != null
										&& !XAffectWindowControl.this.controlInfo.isDisposed())
								{
									XAffectWindowControl.this.controlInfo.setEnabled(false);
								}
							}
						});
						break;
					default:
						// do nothing
				}
				// log status-update
				XAffectWindowControl.this.log("System reached status \"" + (State) evt.getNewValue() + "\".");
			}
		});

		// watch allowMeasuring-listener and change btnStartMeasuring adequately
		super.getLifeCycleManager().registerAllowMeasuringChangedListener(new PropertyChangeListener()
		{
			@Override
			public void propertyChange(final PropertyChangeEvent evt)
			{
				XAffectWindowControl.this.display.syncExec(new Runnable()
				{

					@Override
					public void run()
					{
						if ((Boolean) evt.getNewValue() && XAffectWindowControl.this.controlInfo != null
								&& !XAffectWindowControl.this.controlInfo.isDisposed())
						{
							XAffectWindowControl.this.controlInfo.enableMeasuring();
						}
					}
				});
			}
		});

		// watch init-progress and update progressBar adequately
		super.getLifeCycleManager().registerConfigStatusChangedListener(new PropertyChangeListener()
		{
			@Override
			public void propertyChange(final PropertyChangeEvent evt)
			{
				XAffectWindowControl.this.updateProgressbar(true, (Integer) evt.getNewValue());
			}
		});
	}

	/**
	 * Checks for configuration-consistency and configures the
	 * {@link LifeCycleManager} if everything is okay.
	 */
	protected void configure()
	{
		String message;

		// check for missing subject information
		if (this.subjectInfo.getSubjectID().isEmpty() || this.subjectInfo.getSessionID().isEmpty()
				|| this.subjectInfo.getInstitutionID().isEmpty() || this.subjectInfo.getStudyID().isEmpty())
		{
			// Check subject information
			MessageBox missingSubjectInformation = new MessageBox(this.shell, SWT.OK | SWT.ICON_WARNING);
			missingSubjectInformation.setText("Missing information");
			missingSubjectInformation.setMessage("Please complete subject information!");
			missingSubjectInformation.open();
		}

		// check for Setup-selection
		else if (this.configurationInfo.getSetup() == null)
		{
			// check setup
			MessageBox missingSetupInformation = new MessageBox(this.shell, SWT.OK | SWT.ICON_WARNING);
			missingSetupInformation.setText("Missing information");
			missingSetupInformation.setMessage("Please select Setup!");
			missingSetupInformation.open();
		}

		// check for setup-consistency
		else if (this.configurationInfo.getSetup().checkParameters(this.configurationInfo.getParameters()) != null
				&& !(message = this.configurationInfo.getSetup()
						.checkParameters(this.configurationInfo.getParameters())).isEmpty())
		{
			// Check parameter
			MessageBox missingSetupInformation = new MessageBox(this.shell, SWT.OK | SWT.ICON_WARNING);
			missingSetupInformation.setText("Invalid parameter");
			missingSetupInformation.setMessage(message);
			missingSetupInformation.open();
		}

		// everything is okay
		else
		{
			// disable widgets
			this.display.syncExec(new Runnable()
			{
				@Override
				public void run()
				{
					if (XAffectWindowControl.this.subjectInfo != null
							&& !XAffectWindowControl.this.subjectInfo.isDisposed()
							&& XAffectWindowControl.this.configurationInfo != null
							&& !XAffectWindowControl.this.configurationInfo.isDisposed())
					{
						XAffectWindowControl.this.configurationInfo.setEnabled(false);
						XAffectWindowControl.this.subjectInfo.setEnabled(false);
					}
				}
			});

			// configure LifeCycleManager
			super.getLifeCycleManager().initiate(this.subjectInfo.getSubjectID(), this.subjectInfo.getSessionID(),
					this.subjectInfo.getInstitutionID(), this.subjectInfo.getStudyID());
			super.getLifeCycleManager().configure(this.configurationInfo.getSetup(),
					this.configurationInfo.getParameters());

			this.log("Connected.");
		}
	}

	/**
	 * Updates the {@link XAffectWindowControl}'s {@link ProgressBar}.
	 * 
	 * @param visible
	 *            whether the bar is visible or not.
	 * @param progress
	 *            a percentage to be displayed as progress.
	 */
	private void updateProgressbar(final boolean visible, final int progress)
	{
		this.display.syncExec(new Runnable()
		{
			@Override
			public void run()
			{
				XAffectWindowControl.this.progressBar.setVisible(visible);
				XAffectWindowControl.this.progressBar.setSelection(progress);
			}
		});
	}


	/**
	 * Logs messages in {@link XAffectWindowControl}'s logging-field.
	 * 
	 * @param text
	 *            to be logged.
	 */
	private void log(final String text)
	{
		this.display.syncExec(new Runnable()
		{
			@Override
			public void run()
			{
				XAffectWindowControl.this.textLogging.append("\n"
						+ String.format("%02d", Calendar.getInstance().get(Calendar.HOUR_OF_DAY)) + ":"
						+ String.format("%02d", Calendar.getInstance().get(Calendar.MINUTE)) + ":"
						+ String.format("%02d", Calendar.getInstance().get(Calendar.SECOND)) + " | " + text);
			}
		});
	}

	@Override
	public void open()
	{
		if (this.shell != null)
		{
			this.shell.open();
			this.shell.layout();

			while (!this.shell.isDisposed())
			{
				if (!this.display.readAndDispatch())
				{
					this.display.sleep();
				}
			}
		}
	}

	@Override
	public void close()
	{
		if (this.shell != null && !this.shell.isDisposed())
		{
			this.shell.close();
			this.shell.dispose();
		}
	}
}
