/*
xAffect - Middleware for Physiological Sensors and Affective Signal Processing
Copyright (c) 2011-2012 FZI Research Center for Information Technology, Germany

This file is part of the xAffect middleware. For more information, see http://www.xaffect.org.

This software is licensed under the Original BSD License. You should have received a copy of the Original BSD License along with xAffect. If not, see http://www.xaffect.org/license/.
 */

package org.xaffect.desktop.classloader;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.net.MalformedURLException;
import java.net.URL;
import java.security.CodeSource;
import java.security.SecureClassLoader;
import java.security.cert.Certificate;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;

/**
 * Loads {@link Class}es from JARs in xAffect's running JAR's directory.<br />
 * Extends {@link ClassLoader} as a PARENT_LAST-Loader, which means, that the
 * parent ClassLoader is only consulted, if this loader did not find any results
 * on its own.
 */
public class DynamicClassLoader extends SecureClassLoader
{
	/**
	 * Array of directories not to be crawled.
	 */
	private final String[] BLACKLIST;
	/**
	 * The root-directory of the application.
	 */
	private File root;
	/**
	 * {@link Hashtable} containing all classes that were found (see
	 * {@link #findClass(String)}) yet.
	 */
	private Hashtable<String, Class<?>> cachedClasses = new Hashtable<String, Class<?>>();
	/**
	 * {@link Hashtable} containing all indexed classes.
	 */
	private Hashtable<String, Resource> foundClasses = new Hashtable<String, Resource>();
	/**
	 * {@link Hashtable} containing all indexed resources.
	 */
	private Hashtable<String, Resource> foundResources = new Hashtable<String, Resource>();
	/**
	 * {@link Hashtable} containing all indexed native libraries.
	 */
	private Hashtable<String, ResourceFromFile> foundNatives = new Hashtable<String, ResourceFromFile>();

	/**
	 * Creates a new {@link DynamicClassLoader}.
	 * 
	 * @param debug
	 *            will create a dump-file in root-directory if true.
	 */
	public DynamicClassLoader(boolean debug)
	{
		super(DynamicClassLoader.class.getClassLoader());
		
		//get root-directory of application
		String rootPath = DynamicClassLoader.class.getProtectionDomain().getCodeSource().getLocation().getPath();
		rootPath = rootPath.replaceAll("%20", " ");
		this.root = new File(rootPath).getParentFile();
		
		//create blacklist in which nothing is to be cached
		this.BLACKLIST = new String[] { ".svn", "doc", "log" };

		//crawl for resources
		this.crawlDirectory(this.root, true);
		
		//dump report in debug-mode
		if (debug)
		{
			this.dumpFoundFiles();
		}
	}

	@Override
	public Class<?> loadClass(String className) throws ClassNotFoundException
	{
		Class<?> result = null;

		// try to find class by itself
		try
		{
			result = this.findClass(className);
		}
		catch (ClassNotFoundException ex)
		{
			// if not able to find class by itself
			// try to delegate to parent loader
			result = super.loadClass(className);
			this.cachedClasses.put(className, result);
		}

		return result;
	}

	@Override
	public Class<?> findClass(String className) throws ClassNotFoundException
	{
		// check whether class was already found and cached
		if (this.cachedClasses.containsKey(className))
		{
			return (Class<?>) this.cachedClasses.get(className);
		}
		else if (this.foundClasses.containsKey(className))
		{
			Class<?> result = null;
			Resource foundClass = this.foundClasses.get(className);
			// file was found directly on file-system
			if (foundClass instanceof ResourceFromFile)
			{
				result = this.defineClassFromFile(className, (ResourceFromFile) foundClass);
			}
			// file was found inside a .jar-file
			else if (foundClass instanceof ResourceFromJar)
			{
				result = this.defineClassFromJarEntry(className, (ResourceFromJar) foundClass);
			}

			if (result != null)
			{
				this.cachedClasses.put(className, result);
				return result;
			}
		}
		// throw ClassNotFoundException if no class could be found by now.
		throw new ClassNotFoundException();
	}

	/**
	 * Defines given ResourceFromFile as a class with given className.
	 * 
	 * @param className
	 *            representing the name the resulting class is supposed to have.
	 * @param res
	 *            {@link ResourceFromFile} representing the class to be defined.
	 * @return defined {@link Class} if possible, null otherwise.
	 */
	private Class<?> defineClassFromFile(String className, ResourceFromFile res)
	{
		try
		{
			File clazz = (File) res.getFile();
			InputStream is = new FileInputStream(clazz);
			ByteArrayOutputStream byteStream = new ByteArrayOutputStream();
			int nextValue = is.read();
			while (-1 != nextValue)
			{
				byteStream.write(nextValue);
				nextValue = is.read();
			}
			is.close();
			byte[] classByte = byteStream.toByteArray();
			CodeSource source = new CodeSource(clazz.toURI().toURL(), new Certificate[] {});
			return defineClass(className, classByte, 0, classByte.length, source);
		}
		catch (Exception e)
		{
			return null;
		}
	}

	/**
	 * Defines given ResourceFromJar as a class with given className.
	 * 
	 * @param className
	 *            representing the name the resulting class is supposed to have.
	 * @param res
	 *            {@link ResourceFromJar} representing the class to be defined.
	 * @return defined {@link Class} if possible, null otherwise.
	 */
	private Class<?> defineClassFromJarEntry(String className, ResourceFromJar res)
	{
		try
		{
			JarFile jar = new JarFile(res.getFile());
			InputStream is = jar.getInputStream(((ResourceFromJar) res).getEntry());
			ByteArrayOutputStream byteStream = new ByteArrayOutputStream();
			int nextValue = is.read();
			while (-1 != nextValue)
			{
				byteStream.write(nextValue);
				nextValue = is.read();
			}
			jar.close();
			byte[] classByte = byteStream.toByteArray();
			CodeSource source = new CodeSource(res.file.toURI().toURL(), new Certificate[] {});
			return defineClass(className, classByte, 0, classByte.length, source);
		}
		catch (Exception e)
		{
			return null;
		}
	}

	@Override
	public URL getResource(String name)
	{
		// get fileName
		String fileName = name;
		if (name.contains("/"))
		{
			fileName = name.substring(name.lastIndexOf('/') + 1);
		}

		URL urlToResource = null;

		// check if fileName was indexed
		if (this.foundResources.containsKey(fileName))
		{
			// strip ./ & ../
			String semiAbsolutePath = name.replaceAll("\\./", "").replaceAll("\\.\\./", "");
			Resource resource = this.foundResources.get(fileName);

			// check for matching path
			do
			{
				try
				{
					if (resource instanceof ResourceFromFile)
					{
						if (resource.getFile().getAbsolutePath().endsWith(semiAbsolutePath))
						{
							urlToResource = resource.getFile().toURI().toURL();
						}
					}
					else if (resource instanceof ResourceFromJar)
					{
						if (((ResourceFromJar) resource).getEntry().getName().endsWith(semiAbsolutePath))
						{
							urlToResource = new URL("jar:" + resource.getFile().toURI().toURL() + "!/"
									+ ((ResourceFromJar) resource).getEntry().getName());
						}
					}
				}
				catch (MalformedURLException ex)
				{

				}
			} while (urlToResource == null && (resource = resource.getNext()) != null);
		}

		// if resource was not found consult bootstrap-loader
		if (urlToResource == null)
		{
			urlToResource = super.findResource(name);
		}

		return urlToResource;
	}

	@Override
	public String findLibrary(String libname)
	{
		if (this.foundNatives.containsKey(libname + ".dll"))
		{
			return this.foundNatives.get(libname + ".dll").file.getAbsolutePath();
		}
		else
		{
			return super.findLibrary(libname);
		}
	}

	/**
	 * Crawls given directory for classes and resources and indexes them.
	 * 
	 * @param directory
	 *            {@link File} representing a directory to be crawled.
	 * @param recursively
	 *            whether sub-directories are to be crawled also.
	 */
	private void crawlDirectory(File directory, boolean recursively)
	{
		if (directory.exists())
		{
			// file is a directory
			if (directory.isDirectory() && recursively)
			{
				boolean isOnBlacklist = false;
				for (String folder : this.BLACKLIST)
				{
					if (folder.equalsIgnoreCase(directory.getName()))
					{
						isOnBlacklist = true;
						break;
					}
				}

				if (!isOnBlacklist)
				{
					// crawl all files in directory
					for (File subFile : directory.listFiles())
					{
						crawlDirectory(subFile, true);
					}
				}
			}
			// file is a file
			else
			{
				indexFile(directory);
			}
		}
	}

	/**
	 * Crawls a .jar-file for resources and indexes them.<br />
	 * If the .jar-file contains a nested .jar-file, it is unziped to the parent
	 * directory. If the parent directory already contains a file with the
	 * nested .jar-file's name, nothing happens.
	 * 
	 * @param jarFile .jar to crawl
	 */
	private void crawlJar(File jarFile)
	{
		try
		{
			JarFile jar = new JarFile(jarFile);
			Enumeration<JarEntry> entries = jar.entries();
			JarEntry entry;
			while (entries.hasMoreElements())
			{
				entry = entries.nextElement();
				if (!entry.isDirectory())
				{
					if (entry.getName().endsWith(".class"))
					{
						this.indexClassFromJar(jarFile, entry);
					}
					else if (entry.getName().endsWith(".jar"))
					{
						// unzip jar-file
						File dirToUnzipTo = jarFile.getParentFile();
						File fileToUnzipTo = new File(dirToUnzipTo, this.stripNameOfJarEntry(entry));

						if (!fileToUnzipTo.exists())
						{
							try
							{
								fileToUnzipTo.createNewFile();
								URL urlToEntry = new URL("jar:" + jarFile.toURI().toURL() + "!/" + entry.getName());
								InputStream in = urlToEntry.openStream();
								FileOutputStream out = new FileOutputStream(fileToUnzipTo);
								byte[] buffer = new byte[1024];
								int len;
								while ((len = in.read(buffer, 0, buffer.length)) != -1)
								{
									out.write(buffer, 0, len);
								}
								in.close();
								out.close();
								this.crawlJar(fileToUnzipTo);
							}
							catch (MalformedURLException e)
							{
								// don't consider non-readable file as an option
							}
						}
					}
					else
					{
						this.indexResourceFromJar(jarFile, entry);
					}
				}
			}
			jar.close();
		}
		catch (IOException e)
		{
			// don't consider non-readable file as an option
		}
	}

	private String stripNameOfJarEntry(JarEntry entry)
	{
		String entryName = entry.getName();
		int beginIndex = entryName.lastIndexOf('/');
		if (beginIndex > 0)
		{
			return entryName.substring(beginIndex, entryName.length());
		}
		else
		{
			return entryName;
		}
	}

	/**
	 * Classifies given {@link File} and indexes it in it's corresponding
	 * {@link Hashtable}.
	 * 
	 * @param file
	 *            {@link File} to index.
	 */
	private void indexFile(File file)
	{
		// classify file:
		if (file.getName().endsWith(".class"))
		{
			this.indexClass(file);
		}
		else if (file.getName().endsWith(".dll"))
		{
			this.indexLibrary(file);
		}
		else if (file.getName().endsWith(".jar") || file.getName().endsWith(".JAR"))
		{
			this.crawlJar(file);
		}
		// everything else will be interpreted as a resource
		else
		{
			this.indexResource(file);
		}
	}

	/**
	 * Indexes given file as a class.
	 * 
	 * @param file
	 *            class to be indexed.
	 */
	private void indexClass(File file)
	{
		// get package-name starting with "org"
		if (file.getPath().contains("org\\"))
		{
			String binaryName = file.getPath().substring(file.getPath().indexOf("org\\"));
			// replace path-separator with package-separator
			binaryName = binaryName.replace("\\", ".");
			// strip ".class"
			binaryName = binaryName.substring(0, binaryName.lastIndexOf('.'));
			this.foundClasses.put(binaryName, new ResourceFromFile(file));
		}
	}

	/**
	 * Indexes given entry as a class placed in given jar-file.<br />
	 * Directories inside the jar are considered packages beginning at the jar's
	 * root-directory.
	 * 
	 * @param jar
	 *            JAR-{@link File} the class is encapsulated in.
	 * @param entry
	 *            {@link JarEntry} representing the class.
	 */
	private void indexClassFromJar(File jar, JarEntry entry)
	{
		String binaryName = entry.getName();
		// replace path-separator with package-separator
		binaryName = binaryName.replace('/', '.');
		// strip ".class"
		binaryName = binaryName.substring(0, binaryName.lastIndexOf('.'));
		this.foundClasses.put(binaryName, new ResourceFromJar(jar, entry));
	}

	/**
	 * Indexes given file as a native library.<b />
	 * 
	 * @param file
	 *            {@link File} representing the library to be indexed.
	 */
	private void indexLibrary(File file)
	{
		String name = file.getName();
		name = name.replaceAll(".*/", "");
		this.foundNatives.put(name, new ResourceFromFile(file));
	}

	/**
	 * Indexes given file as a resource.
	 * 
	 * @param file
	 *            {@link File} representing the resource to be indexed.
	 */
	private void indexResource(File file)
	{
		// index found resource
		this.indexResource(new ResourceFromFile(file), file.getName());
	}

	/**
	 * Indexes given entry as a resource encapsulated in given jar.
	 * 
	 * @param jar
	 *            JAR-{@link File} the resource is encapsulated in.
	 * @param entry
	 *            representing the resource to be indexed.
	 */
	private void indexResourceFromJar(File jar, JarEntry entry)
	{
		String fileName = entry.getName();
		if (entry.getName().contains("/"))
		{
			fileName = entry.getName().substring(entry.getName().lastIndexOf('/') + 1);
		}
		this.indexResource(new ResourceFromJar(jar, entry), fileName);
	}

	/**
	 * Indexes given resource with given fileName.<br />
	 * Will chain resources if they already exist in {@link #foundResources}.
	 * 
	 * @param res
	 *            {@link Resource} to be indexed.
	 * @param fileName
	 *            hash to be used.
	 */
	private void indexResource(Resource res, String fileName)
	{
		// index found resource
		if (!this.foundResources.contains(fileName))
		{
			// fileName not yet indexed -> index
			this.foundResources.put(fileName, res);
		}
		else
		{
			// fileName indexed -> append to last chained
			// Resource
			Resource resource = this.foundResources.get(fileName);
			while (resource.getNext() != null)
			{
				resource = resource.getNext();
			}
			resource.setReource(res);
		}
	}

	/**
	 * Creates a .txt-file in the project's root-directory, in which all found
	 * classes and resources are listed. [for debugging]
	 */
	public void dumpFoundFiles()
	{
		try
		{
			String path = this.root.getAbsolutePath() + "/ClassLoaderDump.txt";
			System.out.println("Dumping report for classloader at '" + path + "'");
			FileWriter fstream = new FileWriter(path);
			PrintWriter out = new PrintWriter(fstream);

			out.println("#################################CLASSES#################################");

			Enumeration<String> classKeys = this.foundClasses.keys();
			while (classKeys.hasMoreElements())
			{
				out.println(classKeys.nextElement());
			}

			out.println("################################RESOURCES################################");

			Enumeration<String> resourceKeys = this.foundResources.keys();
			while (resourceKeys.hasMoreElements())
			{
				out.println(resourceKeys.nextElement());
			}

			out.println("#################################NATIVES#################################");
			Enumeration<String> nativeKeys = this.foundNatives.keys();
			while (nativeKeys.hasMoreElements())
			{
				out.println(nativeKeys.nextElement());
			}

			out.close();
		}
		catch (Exception e)
		{
			System.out.println("Dumping report failed: " + e.getMessage());
		}
	}
}
