/*
xAffect - Middleware for Physiological Sensors and Affective Signal Processing
Copyright (c) 2011-2012 FZI Research Center for Information Technology, Germany

This file is part of the xAffect middleware. For more information, see http://www.xaffect.org.

This software is licensed under the Original BSD License. You should have received a copy of the Original BSD License along with xAffect. If not, see http://www.xaffect.org/license/.
 */

package org.xaffect.desktop.classloader;

import java.io.File;
import java.io.IOException;
import java.util.Enumeration;
import java.util.Vector;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;

/**
 * Util-class to find setups.
 */
public class SetupFinder
{
	/**
	 * Finds setups inside given root-file as local .class-files (setups have to
	 * be placed inside a folder called "setup") or as entries of .JAR-files.
	 * 
	 * @param root
	 *            {@link File} to be searched in.
	 * @return {@link Vector} containing all binary names of found setups.
	 */
	public static Vector<String> getAllSetupsIn(File root)
	{
		Vector<String> result = new Vector<String>();

		try
		{
			// check for setups as local .class-files
			insertAllSetupsInDirectory(result, root);

			// check for setups in .jar-files
			insertAllSetupsFromJARs(result, root);
		}
		catch (Exception ex)
		{
			System.out.println(ex.getMessage());
			ex.printStackTrace();
		}

		return result;
	}

	/**
	 * Searches for setups inside a given directory recursively.
	 * 
	 * @param container
	 *            {@link Vector} found setups are to be stored in.
	 * @param root
	 *            {@link File} to be searched in.
	 */
	private static void insertAllSetupsInDirectory(Vector<String> container, File root)
	{
		if (root.exists() && root.isDirectory())
		{
			String name = root.getName();
			if (name.equalsIgnoreCase("setup"))
			{
				insertSetupsFromSetupFolder(container, root);
			}
			else
			{
				for (File file : root.listFiles())
				{
					if (file.isDirectory())
					{
						insertAllSetupsInDirectory(container, file);
					}
				}
			}
		}
	}

	/**
	 * Assumes given folder is a valid "setup"-folder and tests all found
	 * .class-files via ClassLoader. Valid files are added to given Vector.
	 * 
	 * @param container
	 *            {@link Vector} found setups are to be stored in.
	 * @param root
	 *            {@link File} representing a valid setup-folder.
	 */
	private static void insertSetupsFromSetupFolder(Vector<String> container, File root)
	{
		for (File file : root.listFiles())
		{
			if (file.isFile() && file.getName().endsWith(".class"))
			{
				String className = "org.xaffect.setup." + file.getName().substring(0, file.getName().lastIndexOf("."));
				try
				{
					SetupFinder.class.getClassLoader().loadClass(className);
					container.add(className);
				}
				catch (ClassNotFoundException e)
				{
					// meh
				}
			}
		}
	}

	/**
	 * Searches for setups placed inside of .JAR-files inside a given directory
	 * recursively.
	 * 
	 * @param container
	 *            {@link Vector} found setups are to be stored in.
	 * @param root
	 *            {@link File} to be searched in.
	 */
	private static void insertAllSetupsFromJARs(Vector<String> container, File root)
	{
		if (root.exists() && root.isDirectory())
		{
			for (File file : root.listFiles())
			{
				if (file.isDirectory())
				{
					insertAllSetupsFromJARs(container, file);
				}
				else if (file.getName().endsWith(".jar") || file.getName().endsWith(".JAR"))
				{
					try
					{
						JarFile jar = new JarFile(file);
						Enumeration<JarEntry> entries = jar.entries();
						while (entries.hasMoreElements())
						{
							JarEntry entry = entries.nextElement();
							if (entry.getName().startsWith("org/xaffect/setup") && !entry.isDirectory()
									&& entry.getName() != "org/xaffect/setup/Setup.class")
							{
								container.add(entry.getName().replace('/', '.')
										.substring(0, entry.getName().lastIndexOf('.')));
							}
						}
						jar.close();
					}
					catch (IOException e)
					{
						// don't consider file as an option if it cannot be
						// read.
					}
				}
			}
		}
	}
}
