/*
xAffect - Middleware for Physiological Sensors and Affective Signal Processing
Copyright (c) 2011-2012 FZI Research Center for Information Technology, Germany

This file is part of the xAffect middleware. For more information, see http://www.xaffect.org

This software is licensed under the Original BSD License. You should have received a copy of the Original BSD License along with xAffect. If not, see http://www.xaffect.org/license/.
 */

package org.xaffect.setup;

import java.util.HashMap;

import org.xaffect.components.processor.slidingmean.ProcessorSlidingMean;
import org.xaffect.components.sink.dataviewer.SinkDataViewer;
import org.xaffect.components.sink.unisenslogger.SinkUnisensLogger;
import org.xaffect.components.source.signalgenerators.SourceEcgGenerator;
import org.xaffect.components.source.statuslistener.SourceStatusListener;
import org.xaffect.core.LifeCycleManager;

/**
 * This is an example setup for xAffect. It shows the handling of some standard
 * components.
 * 
 * @author kirst, platzek
 * 
 */
public class ExampleSetupXAffectDesktop extends Setup
{
	// This is an example for a user specific parameter in the GUI.
	private static final String PARAMETER_EXAMPLE_COMMENT = "Example Comment";

	@Override
	public void instantiateComponents(LifeCycleManager manager, HashMap<String, String> parameter) throws Exception
	{
		// In this method the setup is initialized. For every component used, we
		// have a configuration hash map and one line of code for
		// initialization.

		// This initializes the ECG generator. By default it generates a
		// realistic 250 Hz ECG signal.
		HashMap<String, String> cfgGenerator = new HashMap<String, String>();
		cfgGenerator.put(SourceEcgGenerator.CONFIG_CONTENT_CLASS_ECG, "ECG");
		cfgGenerator.put(SourceEcgGenerator.CONFIG_PRODUCE_ECG, "TRUE");
		new SourceEcgGenerator(manager, cfgGenerator);

		// The sliding mean algorithm listens to the generated ECG and
		// calculates a new signal, the filtered ECG with the content class
		// FILTEREDECG.
		HashMap<String, String> cfgSlidingMean = new HashMap<String, String>();
		cfgSlidingMean.put(ProcessorSlidingMean.CONFIG_CONTENT_CLASS_INPUT, "ECG");
		cfgSlidingMean.put(ProcessorSlidingMean.CONFIG_CONTENT_CLASS_OUTPUT, "FILTEREDECG");
		cfgSlidingMean.put(ProcessorSlidingMean.CONFIG_SAMPLE_RATE, "250");
		cfgSlidingMean.put(ProcessorSlidingMean.CONFIG_NUMBER_OF_CHANNELS, "1");
		cfgSlidingMean.put(ProcessorSlidingMean.CONFIG_WINDOW_WIDTH, "7");
		new ProcessorSlidingMean(manager, cfgSlidingMean);

		// Now we are defining two different viewers for our signals. The first
		// one listens to our filtered ECG signal, ...
		HashMap<String, String> cfgDataViewerFiltered = new HashMap<String, String>();
		cfgDataViewerFiltered.put(SinkDataViewer.CONFIG_CONTENT_CLASS, "FILTEREDECG");
		cfgDataViewerFiltered.put(SinkDataViewer.CONFIG_SAMPLE_RATE, "250");
		cfgDataViewerFiltered.put(SinkDataViewer.CONFIG_NUMBER_OF_CHANNELS, "1");
		cfgDataViewerFiltered.put(SinkDataViewer.CONFIG_STYLE, Integer.toString(SinkDataViewer.SIGNAL));
		new SinkDataViewer(manager, cfgDataViewerFiltered);

		// ... the second one to the ECG raw signal.
		HashMap<String, String> cfgDataViewerEcg = new HashMap<String, String>();
		cfgDataViewerEcg.put(SinkDataViewer.CONFIG_CONTENT_CLASS, "ECG");
		cfgDataViewerEcg.put(SinkDataViewer.CONFIG_SAMPLE_RATE, "250");
		cfgDataViewerEcg.put(SinkDataViewer.CONFIG_NUMBER_OF_CHANNELS, "1");
		cfgDataViewerEcg.put(SinkDataViewer.CONFIG_STYLE, Integer.toString(SinkDataViewer.SIGNAL));
		new SinkDataViewer(manager, cfgDataViewerEcg);

		// This component fires an event on each status change (Start Baseline,
		// Start Measuring and Stop Baseline). In this case we use the generated
		// information for logging purposes.
		HashMap<String, String> cfgStatusListener = new HashMap<String, String>();
		cfgStatusListener.put(SourceStatusListener.CONFIG_CONTENT_CLASS, "STATE");
		cfgStatusListener.put(SourceStatusListener.CONFIG_SAMPLE_RATE, "1000");
		new SourceStatusListener(manager, cfgStatusListener);

		// Three data loggers are defined. Every data logger logs in the same
		// data set, because all logging paths are the same. Logging into
		// different files is possible, too. When you don't want to hard code
		// the logging path, you can set it with the setup configuration
		// parameters at the end of this file. The first logger saves the ECG
		// raw signal, ...
		HashMap<String, String> cfgUnisensLogger1 = new HashMap<String, String>();
		cfgUnisensLogger1.put(SinkUnisensLogger.CONFIG_SAMPLE_RATE, "250");
		cfgUnisensLogger1.put(SinkUnisensLogger.CONFIG_LOGGING_PATH, "./unisensFiles");
		cfgUnisensLogger1.put(SinkUnisensLogger.CONFIG_ENTRY_ID, "ecg");
		cfgUnisensLogger1.put(SinkUnisensLogger.CONFIG_CONTENT_CLASS, "ECG");
		cfgUnisensLogger1.put(SinkUnisensLogger.CONFIG_FILE_FORMAT, SinkUnisensLogger.FILE_FORMAT_BIN);
		new SinkUnisensLogger(manager, cfgUnisensLogger1);

		// ... the second one the filtered ECG and ...
		HashMap<String, String> cfgUnisensLogger2 = new HashMap<String, String>();
		cfgUnisensLogger2.put(SinkUnisensLogger.CONFIG_SAMPLE_RATE, "250");
		cfgUnisensLogger2.put(SinkUnisensLogger.CONFIG_LOGGING_PATH, "./unisensFiles");
		cfgUnisensLogger2.put(SinkUnisensLogger.CONFIG_ENTRY_ID, "ecg_filtered");
		cfgUnisensLogger2.put(SinkUnisensLogger.CONFIG_CONTENT_CLASS, "FILTEREDECG");
		cfgUnisensLogger2.put(SinkUnisensLogger.CONFIG_COMMENT, "ECG signal filtered with a sliding mean filter");
		cfgUnisensLogger1.put(SinkUnisensLogger.CONFIG_FILE_FORMAT, SinkUnisensLogger.FILE_FORMAT_BIN);
		new SinkUnisensLogger(manager, cfgUnisensLogger2);

		// ... the last one the status events. You can use the free
		// UnisensViewer (http://unisens.org) for displaying the recorded data
		// sets. You find the files in the created folder unisensFiles in the
		// xAffect base directory. The value of CONFIG_COMMENT is set by the
		// GUI (see getKeys() and checkParameters()).
		HashMap<String, String> cfgUnisensLogger3 = new HashMap<String, String>();
		cfgUnisensLogger3.put(SinkUnisensLogger.CONFIG_SAMPLE_RATE, "1000");
		cfgUnisensLogger3.put(SinkUnisensLogger.CONFIG_LOGGING_PATH, "./unisensFiles");
		cfgUnisensLogger3.put(SinkUnisensLogger.CONFIG_ENTRY_ID, "state");
		cfgUnisensLogger3.put(SinkUnisensLogger.CONFIG_CONTENT_CLASS, "STATE");
		cfgUnisensLogger3.put(SinkUnisensLogger.CONFIG_EVENT_COMMENT_LENGTH, "31");
		cfgUnisensLogger3.put(SinkUnisensLogger.CONFIG_EVENT_TYPE_LENGTH, "9");
		cfgUnisensLogger3.put(SinkUnisensLogger.CONFIG_FILE_FORMAT, SinkUnisensLogger.FILE_FORMAT_CSV);
		cfgUnisensLogger3.put(SinkUnisensLogger.CONFIG_COMMENT, parameter.get(PARAMETER_EXAMPLE_COMMENT));
		new SinkUnisensLogger(manager, cfgUnisensLogger3);
	}

	@Override
	public String[] getKeys()
	{
		// You can define setup specific key-value-pairs. They will be shown in
		// the GUI or can be set via network control. When you don't need any
		// parameters, just return null. Besides a properties file, the
		// parameters are a good way to set user specific parameters.
		return new String[] { PARAMETER_EXAMPLE_COMMENT };
	}

	@Override
	public String checkParameters(HashMap<String, String> parameter)
	{
		// If you defined some keys above, you will check the parameters. In
		// the example case, we check whether the key PARAMETER_EXAMPLE_COMMENT
		// is not empty. But you can use regular expressions as well, or just
		// return null, when you don't have any parameters or you don't want to
		// check them. The return message is shown in a message box.
		if (parameter.containsKey(PARAMETER_EXAMPLE_COMMENT))
		{
			if (parameter.get(PARAMETER_EXAMPLE_COMMENT).isEmpty())
			{
				return "The parameter '" + PARAMETER_EXAMPLE_COMMENT + "' is missing.";
			}
		}

		return null;
	}

}
